// Copyright (C) 2005 CEREA
//     Author: Vivien Mallet
//
// CEREA (http://www.enpc.fr/cerea) is a joint laboratory of
// ENPC (http://www.enpc.fr) and EDF R&D (http://www.edf.fr).
//
// This file is part of the chemistry-transport model Polair3D.
//
// This code is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This file is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License (file "license") for more details.


#ifndef POLYPHEMUS_FILE_DRIVER_BASEDRIVER_CXX


#include "BaseDriver.hxx"


namespace Polyphemus
{


  //! Main constructor.
  /*! Builds the driver and reads option keys in the configuration file.
    \param config_file configuration file.
   */
  template<class T, class ClassModel, class ClassOutputSaver>
  BaseDriver<T, ClassModel, ClassOutputSaver>::BaseDriver(string config_file):
    Model(config_file), config(config_file)
  {

    /*** Display options ***/

    config.SetSection("[display]");
    // Should iterations be displayed on screen?
    config.PeekValue("Show_iterations", option_display["show_iterations"]);
    // Should current date be displayed on screen?
    config.PeekValue("Show_date", option_display["show_date"]);
  }

  
  //! Destructor.
  template<class T, class ClassModel, class ClassOutputSaver>
  BaseDriver<T, ClassModel, ClassOutputSaver>::~BaseDriver()
  {
  }


  //! Performs the simulation.
  /*! Initializes the model and the output saver, and then performs the time
    loop with calls to the model and to the output saver.
   */
  template<class T, class ClassModel, class ClassOutputSaver>
  void BaseDriver<T, ClassModel, ClassOutputSaver>::Run()
  {

    /*** Initializations ***/

    Model.Init();
    OutputSaver.Init(Model);

    /*** Time loop ***/

    for (int i = 0; i < Model.GetNt(); i++)
      {
	if (option_display["show_iterations"])
	  cout << "Performing iteration #" << i << endl;

	if (this->option_display["show_date"])
	  cout << "Current date: " <<
	    this->Model.GetCurrentDate().GetDate("%y-%m-%d %h:%i") << endl;

	Model.InitStep();
	OutputSaver.InitStep(Model);

	Model.Forward();
	OutputSaver.Save(Model);
      }
  }


} // namespace Polyphemus.


#define POLYPHEMUS_FILE_DRIVER_BASEDRIVER_CXX
#endif
