// Copyright (C) 2005 CEREA
//     Author: Vivien Mallet
//
// CEREA (http://www.enpc.fr/cerea) is a joint laboratory of
// ENPC (http://www.enpc.fr) and EDF R&D (http://www.edf.fr).
//
// This file is part of the chemistry-transport model Polair3D.
//
// This code is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This file is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License (file "license") for more details.


#ifndef POLYPHEMUS_FILE_DRIVER_BASEDRIVER_HXX


#include <vector>
#include <string>
#include <map>
#include "AtmoData.hxx"


namespace Polyphemus
{


  using namespace std;
  using namespace AtmoData;


  ////////////////
  // BASEDRIVER //
  ////////////////


  //! Simple driver that performs a forward time-integration.
  /*! The driver is responsible for the model initialization, for the time
    loop and for the calls to the output saver. Its reference floating-point
    precision is 'T'. The model is an instance of 'ClassModel' and the output
    saver is an instance of 'ClassOutputSaver'.
   */
  template<class T, class ClassModel, class ClassOutputSaver>
  class BaseDriver
  {

  protected:

    /*** Main components ***/

    //! Underlying model.
    ClassModel Model;
    //! Output saver.
    ClassOutputSaver OutputSaver;

    /*** Configuration ***/

    //! Configuration stream.
    ConfigStream config;
    //! Display options.
    map<string, bool> option_display;

  public:

    /*** Constructors and destructor ***/

    BaseDriver(string config_file);
    virtual ~BaseDriver();
    virtual void Run();

  };


} // namespace Polyphemus.


#define POLYPHEMUS_FILE_DRIVER_BASEDRIVER_HXX
#endif
