// Copyright (C) 2006 CEREA
//     Author: Vivien Mallet
//
// CEREA (http://www.enpc.fr/cerea) is a joint laboratory of
// ENPC (http://www.enpc.fr) and EDF R&D (http://www.edf.fr).
//
// This file is part of the modeling system Polyphemus.
//
// This code is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This file is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License (file "license") for more details.


#ifndef POLYPHEMUS_FILE_DRIVER_STATIONARYDRIVER_HXX


#include "BaseDriver.cxx"
#include <map>
#include <vector>
#include <string>
#include "AtmoData.hxx"


namespace Polyphemus
{


  using namespace std;
  using namespace AtmoData;


  //////////////////////
  // STATIONARYDRIVER //
  //////////////////////


  //! Simple driver that performs a forward time-integration at local scale.
  /*! The driver is responsible for the model initialization, for the time
    loop and for the calls to the output saver. Its reference floating-point
    precision is 'T'. The model is an instance of 'ClassModel' and the output
    saver is an instance of 'ClassOutputSaver'.
   */
  template<class T, class ClassModel, class ClassOutputSaver>
  class StationaryDriver: public BaseDriver<T, ClassModel, ClassOutputSaver>
  {

  protected:

    /*** Stationary conditions ***/

    //! Number of stationary steps.
    int Nt;
    //! Time step between stationary steps.
    T Delta_t;

  public:

    /*** Constructors and destructor ***/

    StationaryDriver(string config_file);
    virtual ~StationaryDriver();
    virtual void Run();

  };


} // namespace Polyphemus.


#define POLYPHEMUS_FILE_DRIVER_STATIONARYDRIVER_HXX
#endif
