// Copyright (C) 2005 CEREA
//     Author: Vivien Mallet
//
// CEREA (http://www.enpc.fr/cerea) is a joint laboratory of
// ENPC (http://www.enpc.fr) and EDF R&D (http://www.edf.fr).
//
// This file is part of the chemistry-transport model Polair3D.
//
// This code is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This file is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License (file "license") for more details.


#ifndef POLYPHEMUS_FILE_OUTPUT_SAVER_BASEOUTPUTSAVER_HXX


#include "SaverUnitDomain.cxx"
#include "SaverUnitDomain_aer.cxx"
#include "SaverUnitDomain_assimilation.cxx"
#include "SaverUnitNesting.cxx"
#include "SaverUnitNesting_aer.cxx"
#include "SaverUnitSubdomain.cxx"
#include "SaverUnitSubdomain_aer.cxx"


namespace Polyphemus
{


  //////////////
  // INCLUDES //
  //////////////


#include <vector>
#include <fstream>
  using namespace std;

#include "AtmoData.hxx"
  using namespace AtmoData;


  /////////////////////
  // BASEOUTPUTSAVER //
  /////////////////////


  //! This class manages a set of output-saver units.
  template<class T, class ClassModel>
  class BaseOutputSaver
  {

  protected:

    //! List of output-saver units.
    vector<BaseSaverUnit<T, ClassModel>*> Units;

    //! Group of active saver units.
    string group;

  public:

    BaseOutputSaver();
    ~BaseOutputSaver();

    void SetGroup(string grp);
    string GetGroup() const;
    
    void Init(ClassModel& Model);
    void InitStep(ClassModel& Model);
    void Save(ClassModel& Model);

  };


} // namespace Polyphemus.


#define POLYPHEMUS_FILE_OUTPUT_SAVER_BASEOUTPUTSAVER_HXX
#endif
