// Copyright (C) 2005 CEREA
//     Author: Vivien Mallet, Lin Wu
//
// CEREA (http://www.enpc.fr/cerea) is a joint laboratory of
// ENPC (http://www.enpc.fr) and EDF R&D (http://www.edf.fr).
//
// This file is part of the chemistry-transport model Polair3D.
//
// This code is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This file is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License (file "license") for more details.


#ifndef POLYPHEMUS_FILE_OUTPUT_SAVER_SAVERUNITDOMAIN_ASSIMILATION_CXX


#include "SaverUnitDomain_assimilation.hxx"


namespace Polyphemus
{


  //////////////////////////////////
  // SAVERUNITDOMAIN_ASSIMILATION //
  //////////////////////////////////


  //! Main constructor.
  template<class T, class ClassModel>
  SaverUnitDomain_assimilation<T, ClassModel>
  ::SaverUnitDomain_assimilation():SaverUnitDomain<T, ClassModel>()
  {
  }


  //! Destructor.
  template<class T, class ClassModel>
  SaverUnitDomain_assimilation<T, ClassModel>::~SaverUnitDomain_assimilation()
  {
  }

  //! First initialization.
  /*! Reads the configuration.
    \param config_stream configuration stream.
    \param Model model.
   */
  template<class T, class ClassModel>
  void SaverUnitDomain_assimilation<T, ClassModel>
  ::Init(ConfigStream& config_stream, ClassModel& Model)
  {
    SaverUnitDomain<T, ClassModel>::Init(config_stream, Model);

    // Date file name.
    date_file = config_stream.GetValue("Date_file");
    // Empties file.
    ofstream(date_file.c_str()).close();
  }

  //! Type of saver.
  /*!
    \return The string "domain_assimilation".
   */
  template<class T, class ClassModel>
  string SaverUnitDomain_assimilation<T, ClassModel>::GetType()  const
  {
    return "domain_assimilation";
  }


  //! Group of the saver unit.
  /*!
    \return The group of the saver unit, that is, "analysis".
   */
  template<class T, class ClassModel>
  string SaverUnitDomain_assimilation<T, ClassModel>::GetGroup()  const
  {
    return "analysis";
  }


  //! Saves concentrations if needed.
  /*!
    \param Model model with the following interface:
    <ul>
    <li> GetConcentration()
    </ul>
   */
  template<class T, class ClassModel>
  void SaverUnitDomain_assimilation<T, ClassModel>::Save(ClassModel& Model)
  {
    SaverUnitDomain<T, ClassModel>::Save(Model);

    ofstream out_file(date_file.c_str(), ios::app);
    out_file << Model.GetCurrentDate().GetDate("%y-%m-%d_%hh%i %s\n");
    out_file.close();
  }


} // namespace Polyphemus.


#define POLYPHEMUS_FILE_OUTPUT_SAVER_SAVERUNITDOMAIN_ASSIMILATION_CXX
#endif
