// Copyright (C) 2005 CEREA
//     Author: Vivien Mallet
//
// CEREA (http://www.enpc.fr/cerea) is a joint laboratory of
// ENPC (http://www.enpc.fr) and EDF R&D (http://www.edf.fr).
//
// This file is part of the chemistry-transport model Polair3D.
//
// This code is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This file is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License (file "license") for more details.


#ifndef POLYPHEMUS_FILE_OUTPUT_SAVER_SAVERUNITSUBDOMAIN_AER_HXX


#include "BaseSaverUnit.cxx"


namespace Polyphemus
{


  //////////////
  // INCLUDES //
  //////////////


#include <vector>
#include <fstream>
  using namespace std;

#include "AtmoData.hxx"
  using namespace AtmoData;


  /////////////////////////////
  // SAVERUNITSUBDOMAIN_AER //
  ///////////////////////////


  /*! \brief This class is used to save concentrations over a subdomain
    of the underlying model. Chemical species and vertical levels may be
    selected. */
  template<class T, class ClassModel>
  class SaverUnitSubdomain_aer: public BaseSaverUnit<T, ClassModel>
  {

  protected:

    int i_min;
    int i_max;
    int j_min;
    int j_max;
    
    //! List of vertical levels to be saved.
    vector<int> levels;
    //! Number of levels to be saved.
    int Nlevels;

    //! List of aerosol species (with their bins) to be saved.
    vector<pair<string, vector<int> > > species_list_aer;

    //! List of output files.
    vector<vector<string> > output_file;
    
    /*! Buffer used to compute averaged concentrations.It is first indexed by
      the species name and the bin. */
    vector<vector<Data<T, 3> > > Concentration_;

  public:

    SaverUnitSubdomain_aer();
    virtual ~SaverUnitSubdomain_aer();
    
    virtual string GetType() const;

    virtual void Init(ConfigStream& config_stream, ClassModel& Model);
    virtual void InitStep(ClassModel& Model);
    virtual void Save(ClassModel& Model);

  };


} // namespace Polyphemus.


#define POLYPHEMUS_FILE_OUTPUT_SAVER_SAVERUNITSUBDOMAIN_AER_HXX
#endif
