// Copyright (C) 2005 CEREA
//     Author: Vivien Mallet
//
// CEREA (http://www.enpc.fr/cerea) is a joint laboratory of
// ENPC (http://www.enpc.fr) and EDF R&D (http://www.edf.fr).
//
// This file is part of the chemistry-transport model Polair3D.
//
// This code is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This file is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License (file "license") for more details.


// This code is essentially based on the chemistry-transport model Chimere,
// distributed under GNU GPL -- copyright (C) 2005 Institut Pierre-Simon
// Laplace (CNRS), INERIS, LISA (CNRS).


#ifndef POLYPHEMUS_FILE_MODELS_CASTORCHEMISTRY_HXX


#include <vector>
#include "AtmoData.hxx"
#include "CastorTransport.cxx"


namespace Polyphemus
{


  using namespace std;
  using namespace AtmoData;

  
  /////////////////////
  // CASTORCHEMISTRY //
  /////////////////////
  

  //! This class is a solver for an advection-diffusion-reaction equation.
  /*!  CastorChemistry is a clone of Chimere with respect to transport and
    gaseous chemistry.
   */
  template<class T, class ClassTransport, class ClassChemistry>
  class CastorChemistry:
    public CastorTransport<T, ClassTransport>
  {

  public:

    /*** Type declarations ***/

    typedef typename map<string, InputFiles<T> >::iterator
    input_files_iterator;

  public:
    
    /*** Humidity ***/
    
    //! Specific humidity at current date.
    Data<T, 3> SpecificHumidity;
    //! Specific humidity buffer.
    Data<T, 3> FileSpecificHumidity_i;
    //! Specific humidity buffer.
    Data<T, 3> FileSpecificHumidity_f;

    /*** Attenuation ***/
    
    //! Attenuation coefficients at current date.
    Data<T, 2> Attenuation;
    //! Attenuation buffer.
    Data<T, 2> FileAttenuation_i;
    //! Attenuation buffer.
    Data<T, 2> FileAttenuation_f;

    //! Liquid water content (kg / kg).
    Data<T, 3> LiquidWaterContent;
    //! Liquid water content buffer.
    Data<T, 3> FileLiquidWaterContent_i;
    //! Liquid water content buffer.
    Data<T, 3> FileLiquidWaterContent_f;
    
    /*** Chemical mechanism ***/

    //! Chemical mechanism and chemical numerical scheme.
    ClassChemistry Chemistry_;

  public:
    
    /*** Constructor and destructor ***/
    
    CastorChemistry(string config_file);
    virtual ~CastorChemistry();
    
    /*** Configuration ***/

    void ReadConfiguration();
    virtual void CheckConfiguration();

    int GetSpeciesIndex_ext(string species) const;
    
    /*** Initializations ***/

    void Allocate();
    void Init();
    void InitStep();

    virtual void SetDate(Date date);

    /*** Integration ***/

    void Forward();

    /*** Access methods ***/

  protected:

    virtual void InitAllData();
    
  };


} // namespace Polyphemus.


#define POLYPHEMUS_FILE_MODELS_CASTORCHEMISTRY_HXX
#endif
