// Copyright (C) 2006 CEREA
//     Authors: Hadjira Foudhil, Vivien Mallet, Irne Korsakissok
//
// CEREA (http://www.enpc.fr/cerea) is a joint laboratory of
// ENPC (http://www.enpc.fr) and EDF R&D (http://www.edf.fr).
//
// This file is part of the modeling system Polyphemus.
//
// This code is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This file is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License (file "license") for more details.


#ifndef POLYPHEMUS_FILE_MODELS_GAUSSIANPLUME_HXX


#include "PlumeSource.cxx"
#include <list>
#include "BaseModel.cxx"


namespace Polyphemus
{


  using namespace std;


  ////////////////////
  // GAUSSIAN PLUME //
  ////////////////////


  //! Plume Gaussian model.
  template<class T>
  class GaussianPlume: public BaseModel<T>
  {

  protected:

    //! Land category ("rural" or "urban").
    string land_category;

    //! Time ("night" or "day").
    string day_night;

    //! File storing all source data.
    string file_source;
    //! Source list.
    list<PlumeSource<T> > SourceList;
    //! Number of sources.
    int Nsource;

    //! List of species half-lives (s).
    Array<T, 1> half_life_time;

    //! List of species biological half-lives (s).
    Array<T, 1> biological_half_life_time;

    //! List of species scavenging coefficients ( s^(-1) ).
    Array<T, 1> scavenging_coefficient;

    //! List of species deposition velocities (m/s).
    Array<T, 1> deposition_velocity;
    //! Deposition model ("Chamberlain" or "Overcamp").
    string deposition_model;
    //! Number of points to compute the Chamberlain integral.
    int Nchamberlain;

    //! Temperature of ambient air (Celsius degrees).
    T temperature_;
    //! Angle between positive x-axis and wind, counterclockwise (degrees).
    T wind_angle_;
    //! Wind (m/s).
    T wind_;
    //! Inversion height (m).
    T inversion_height_;

    //! Cosinus of wind angle.
    T cos_angle_;
    //! Sinus of wind angle.
    T sin_angle_;

    //! Stability class (A, B, C, D, E or F).
    string stability_class_;
    //! Stability (corresponding integer: from 0 for A to 5 for F).
    int stability_;

  public:

    /*** Main constructor ***/

    GaussianPlume(string config_file);

    /*** Main methods ***/

    virtual void InitSource(string source_file);
    virtual void Init();
    virtual void InitMeteo(ConfigStream& meteo, bool show_meteo);
    using BaseModel<T>::GetConcentration;
    T GetConcentration(int species, T x, T y, T z);
    virtual void Compute();

  };


} // namespace Polyphemus.


#define POLYPHEMUS_FILE_MODELS_GAUSSIANPLUME_HXX
#endif
