// Copyright (C) 2006 CEREA
//     Authors: Hadjira Foudhil, Vivien Mallet
//
// CEREA (http://www.enpc.fr/cerea) is a joint laboratory of
// ENPC (http://www.enpc.fr) and EDF R&D (http://www.edf.fr).
//
// This file is part of the modeling system Polyphemus.
//
// This code is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This file is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License (file "license") for more details.


#ifndef POLYPHEMUS_FILE_MODELS_PLUMESOURCE_HXX


#include <list>


namespace Polyphemus
{


  using namespace std;


  //////////////////
  // SOURCE PLUME //
  //////////////////


  //! This class stores a source description.
  template<class T>
  class PlumeSource
  {
  protected:

    //! Rate.
    T rate_;
    //! Efflux speed of gases.
    T velocity_;
    //! Temperature.
    T temperature_;
    //! Diameter.
    T diameter_;
    //! Abscissa.
    T x_;
    //! Ordinate.
    T y_;
    //! Height.
    T z_;
    //! Species index.
    int species_index_;

  public:

    /*** Constructor ***/

    PlumeSource(T rate, T velocity, T temperature, T diameter,
		T x, T y, T z, int species_index);
    
    /*** Methods ***/
    
    T GetRate() const;
    T GetVelocity() const;
    T GetTemperature() const;
    T GetDiameter() const;
    T GetX() const;
    T GetY() const;
    T GetZ() const;
    int GetSpeciesIndex() const;

  };


} // namespace Polyphemus.


#define POLYPHEMUS_FILE_MODELS_PLUMESOURCE_HXX
#endif
