C------------------------------------------------------------------------
C     
C     Copyright (C) 2001-2003 CEREA (ENPC)
C     
C     This file is part of Polair.
C     Polair is a 3D eulerian chemistry-transport-model.
C     
C     Polair is free software; you can redistribute it and/or modify
C     it under the terms of the GNU General Public License as published by
C     the Free Software Foundation; either version 2 of the License, or
C     (at your option) any later version.
C     
C     Polair is distributed in the hope that it will be useful,
C     but WITHOUT ANY WARRANTY; without even the implied warranty of
C     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C     GNU General Public License (file "LICENSE") for more details.
C     
C------------------------------------------------------------------------

      SUBROUTINE LU_solve (NGAS, M, x)

C------------------------------------------------------------------------
C     
C     -- DESCRIPTION 
C     
C     This routine solves M * y = x where M is an LU factorization
C     computed by LU_decompose.
C     
C------------------------------------------------------------------------
C     
C     -- INPUT VARIABLES
C     
C     M: LU factorization computed by LU_decompose.
C     
C     -- INPUT/OUTPUT VARIABLES
C     
C     x: on entry, the right-hand side of the equation. On exit,
C     the solution of the equation.
C     
C     -- OUTPUT VARIABLES
C     
C------------------------------------------------------------------------
C     
C     -- REMARKS
C     
C------------------------------------------------------------------------
C     
C     -- MODIFICATIONS
C     
C------------------------------------------------------------------------
C     
C     -- AUTHOR(S)
C     
C------------------------------------------------------------------------

      IMPLICIT NONE

C     -- INCLUDE FILES
C     PARACHEM: parameters for sizes of 'chemical' arrays.

      INTEGER NGAS

      DOUBLE PRECISION M(NGAS,NGAS)
      DOUBLE PRECISION x(NGAS)
      DOUBLE PRECISION temp


C     Forward substitution.

      x(1) = x(1) / M(1, 1)

      x(2) = x(2) / M(2, 2)

      x(3) = x(3) / M(3, 3)

      x(4) = x(4) / M(4, 4)

      x(5) = x(5) / M(5, 5)

      x(6) = x(6) / M(6, 6)

      x(7) = x(7) / M(7, 7)

      temp = M(8, 4) * x(4)
      x(8) = ( x(8) - temp ) / M(8, 8)

      x(9) = x(9) / M(9, 9)

      temp = M(10, 9) * x(9)
      x(10) = ( x(10) - temp ) / M(10, 10)

      x(11) = x(11) / M(11, 11)

      x(12) = x(12) / M(12, 12)

      x(13) = x(13) / M(13, 13)

      x(14) = x(14) / M(14, 14)

      temp = M(15, 14) * x(14)
      x(15) = ( x(15) - temp ) / M(15, 15)

      temp = M(16, 14) * x(14)
      x(16) = ( x(16) - temp ) / M(16, 16)

      temp = M(17, 5) * x(5)
      x(17) = ( x(17) - temp ) / M(17, 17)

      temp = M(18, 6) * x(6)
      x(18) = ( x(18) - temp ) / M(18, 18)

      temp = M(19, 5) * x(5)
      temp = temp + M(19, 6) * x(6)
      x(19) = ( x(19) - temp ) / M(19, 19)

      temp = M(20, 5) * x(5)
      temp = temp + M(20, 6) * x(6)
      x(20) = ( x(20) - temp ) / M(20, 20)

      x(21) = x(21) / M(21, 21)

      x(22) = x(22) / M(22, 22)

      x(23) = x(23) / M(23, 23)

      x(24) = x(24) / M(24, 24)

      x(25) = x(25) / M(25, 25)

      x(26) = x(26) / M(26, 26)

      x(27) = x(27) / M(27, 27)

      x(28) = x(28) / M(28, 28)

      x(29) = x(29) / M(29, 29)

      x(30) = x(30) / M(30, 30)

      x(31) = x(31) / M(31, 31)

      temp = M(32, 17) * x(17)
      temp = temp + M(32, 18) * x(18)
      temp = temp + M(32, 31) * x(31)
      x(32) = ( x(32) - temp ) / M(32, 32)

      temp = M(33, 31) * x(31)
      x(33) = ( x(33) - temp ) / M(33, 33)

      temp = M(34, 7) * x(7)
      temp = temp + M(34, 21) * x(21)
      temp = temp + M(34, 28) * x(28)
      x(34) = ( x(34) - temp ) / M(34, 34)

      x(35) = x(35) / M(35, 35)

      x(36) = x(36) / M(36, 36)

      temp = M(37, 17) * x(17)
      temp = temp + M(37, 18) * x(18)
      temp = temp + M(37, 31) * x(31)
      temp = temp + M(37, 36) * x(36)
      x(37) = ( x(37) - temp ) / M(37, 37)

      x(38) = x(38) / M(38, 38)

      temp = M(39, 17) * x(17)
      x(39) = ( x(39) - temp ) / M(39, 39)

      temp = M(40, 18) * x(18)
      x(40) = ( x(40) - temp ) / M(40, 40)

      temp = M(41, 4) * x(4)
      x(41) = ( x(41) - temp ) / M(41, 41)

      x(42) = x(42) / M(42, 42)

      temp = M(43, 11) * x(11)
      temp = temp + M(43, 14) * x(14)
      temp = temp + M(43, 21) * x(21)
      temp = temp + M(43, 27) * x(27)
      temp = temp + M(43, 28) * x(28)
      temp = temp + M(43, 34) * x(34)
      temp = temp + M(43, 42) * x(42)
      x(43) = ( x(43) - temp ) / M(43, 43)

      temp = M(44, 42) * x(42)
      x(44) = ( x(44) - temp ) / M(44, 44)

      temp = M(45, 14) * x(14)
      temp = temp + M(45, 21) * x(21)
      temp = temp + M(45, 26) * x(26)
      temp = temp + M(45, 28) * x(28)
      temp = temp + M(45, 34) * x(34)
      x(45) = ( x(45) - temp ) / M(45, 45)

      temp = M(46, 28) * x(28)
      temp = temp + M(46, 34) * x(34)
      temp = temp + M(46, 39) * x(39)
      temp = temp + M(46, 40) * x(40)
      x(46) = ( x(46) - temp ) / M(46, 46)

      temp = M(47, 14) * x(14)
      x(47) = ( x(47) - temp ) / M(47, 47)

      temp = M(48, 3) * x(3)
      x(48) = ( x(48) - temp ) / M(48, 48)

      temp = M(49, 4) * x(4)
      x(49) = ( x(49) - temp ) / M(49, 49)

      temp = M(50, 27) * x(27)
      x(50) = ( x(50) - temp ) / M(50, 50)

      temp = M(51, 21) * x(21)
      temp = temp + M(51, 28) * x(28)
      temp = temp + M(51, 34) * x(34)
      x(51) = ( x(51) - temp ) / M(51, 51)

      temp = M(52, 14) * x(14)
      temp = temp + M(52, 21) * x(21)
      temp = temp + M(52, 28) * x(28)
      temp = temp + M(52, 34) * x(34)
      temp = temp + M(52, 47) * x(47)
      temp = temp + M(52, 51) * x(51)
      x(52) = ( x(52) - temp ) / M(52, 52)

      temp = M(53, 14) * x(14)
      temp = temp + M(53, 21) * x(21)
      temp = temp + M(53, 28) * x(28)
      temp = temp + M(53, 34) * x(34)
      temp = temp + M(53, 51) * x(51)
      temp = temp + M(53, 52) * x(52)
      x(53) = ( x(53) - temp ) / M(53, 53)

      temp = M(54, 53) * x(53)
      x(54) = ( x(54) - temp ) / M(54, 54)

      temp = M(55, 26) * x(26)
      x(55) = ( x(55) - temp ) / M(55, 55)

      temp = M(56, 47) * x(47)
      temp = temp + M(56, 51) * x(51)
      temp = temp + M(56, 52) * x(52)
      x(56) = ( x(56) - temp ) / M(56, 56)

      temp = M(57, 14) * x(14)
      temp = temp + M(57, 21) * x(21)
      temp = temp + M(57, 26) * x(26)
      temp = temp + M(57, 27) * x(27)
      temp = temp + M(57, 28) * x(28)
      temp = temp + M(57, 34) * x(34)
      temp = temp + M(57, 52) * x(52)
      temp = temp + M(57, 53) * x(53)
      temp = temp + M(57, 56) * x(56)
      x(57) = ( x(57) - temp ) / M(57, 57)

      temp = M(58, 14) * x(14)
      temp = temp + M(58, 21) * x(21)
      temp = temp + M(58, 26) * x(26)
      temp = temp + M(58, 27) * x(27)
      temp = temp + M(58, 28) * x(28)
      temp = temp + M(58, 34) * x(34)
      temp = temp + M(58, 52) * x(52)
      temp = temp + M(58, 53) * x(53)
      temp = temp + M(58, 56) * x(56)
      temp = temp + M(58, 57) * x(57)
      x(58) = ( x(58) - temp ) / M(58, 58)

      temp = M(59, 14) * x(14)
      temp = temp + M(59, 17) * x(17)
      temp = temp + M(59, 18) * x(18)
      temp = temp + M(59, 21) * x(21)
      temp = temp + M(59, 26) * x(26)
      temp = temp + M(59, 27) * x(27)
      temp = temp + M(59, 28) * x(28)
      temp = temp + M(59, 31) * x(31)
      temp = temp + M(59, 34) * x(34)
      temp = temp + M(59, 37) * x(37)
      temp = temp + M(59, 42) * x(42)
      temp = temp + M(59, 46) * x(46)
      temp = temp + M(59, 52) * x(52)
      temp = temp + M(59, 53) * x(53)
      temp = temp + M(59, 56) * x(56)
      x(59) = ( x(59) - temp ) / M(59, 59)

      temp = M(60, 11) * x(11)
      temp = temp + M(60, 14) * x(14)
      temp = temp + M(60, 21) * x(21)
      temp = temp + M(60, 26) * x(26)
      temp = temp + M(60, 27) * x(27)
      temp = temp + M(60, 28) * x(28)
      temp = temp + M(60, 34) * x(34)
      temp = temp + M(60, 42) * x(42)
      temp = temp + M(60, 46) * x(46)
      temp = temp + M(60, 52) * x(52)
      temp = temp + M(60, 53) * x(53)
      temp = temp + M(60, 56) * x(56)
      temp = temp + M(60, 59) * x(59)
      x(60) = ( x(60) - temp ) / M(60, 60)

      temp = M(61, 11) * x(11)
      temp = temp + M(61, 33) * x(33)
      temp = temp + M(61, 37) * x(37)
      temp = temp + M(61, 39) * x(39)
      temp = temp + M(61, 40) * x(40)
      temp = temp + M(61, 46) * x(46)
      temp = temp + M(61, 52) * x(52)
      temp = temp + M(61, 59) * x(59)
      x(61) = ( x(61) - temp ) / M(61, 61)

      temp = M(62, 56) * x(56)
      temp = temp + M(62, 59) * x(59)
      x(62) = ( x(62) - temp ) / M(62, 62)

      temp = M(63, 14) * x(14)
      temp = temp + M(63, 46) * x(46)
      temp = temp + M(63, 48) * x(48)
      temp = temp + M(63, 49) * x(49)
      temp = temp + M(63, 50) * x(50)
      temp = temp + M(63, 51) * x(51)
      temp = temp + M(63, 52) * x(52)
      temp = temp + M(63, 53) * x(53)
      temp = temp + M(63, 54) * x(54)
      temp = temp + M(63, 56) * x(56)
      temp = temp + M(63, 57) * x(57)
      temp = temp + M(63, 58) * x(58)
      temp = temp + M(63, 59) * x(59)
      temp = temp + M(63, 62) * x(62)
      x(63) = ( x(63) - temp ) / M(63, 63)

      temp = M(64, 14) * x(14)
      temp = temp + M(64, 21) * x(21)
      temp = temp + M(64, 26) * x(26)
      temp = temp + M(64, 28) * x(28)
      temp = temp + M(64, 34) * x(34)
      temp = temp + M(64, 52) * x(52)
      temp = temp + M(64, 53) * x(53)
      temp = temp + M(64, 56) * x(56)
      temp = temp + M(64, 59) * x(59)
      x(64) = ( x(64) - temp ) / M(64, 64)

      temp = M(65, 33) * x(33)
      temp = temp + M(65, 37) * x(37)
      temp = temp + M(65, 39) * x(39)
      temp = temp + M(65, 40) * x(40)
      temp = temp + M(65, 47) * x(47)
      temp = temp + M(65, 48) * x(48)
      temp = temp + M(65, 49) * x(49)
      temp = temp + M(65, 50) * x(50)
      temp = temp + M(65, 51) * x(51)
      temp = temp + M(65, 52) * x(52)
      temp = temp + M(65, 54) * x(54)
      temp = temp + M(65, 55) * x(55)
      temp = temp + M(65, 59) * x(59)
      temp = temp + M(65, 62) * x(62)
      temp = temp + M(65, 64) * x(64)
      x(65) = ( x(65) - temp ) / M(65, 65)

      temp = M(66, 33) * x(33)
      temp = temp + M(66, 37) * x(37)
      temp = temp + M(66, 39) * x(39)
      temp = temp + M(66, 40) * x(40)
      temp = temp + M(66, 41) * x(41)
      temp = temp + M(66, 42) * x(42)
      temp = temp + M(66, 46) * x(46)
      temp = temp + M(66, 52) * x(52)
      temp = temp + M(66, 59) * x(59)
      temp = temp + M(66, 64) * x(64)
      x(66) = ( x(66) - temp ) / M(66, 66)

      temp = M(67, 13) * x(13)
      temp = temp + M(67, 37) * x(37)
      temp = temp + M(67, 46) * x(46)
      temp = temp + M(67, 52) * x(52)
      temp = temp + M(67, 59) * x(59)
      temp = temp + M(67, 61) * x(61)
      temp = temp + M(67, 66) * x(66)
      x(67) = ( x(67) - temp ) / M(67, 67)

      temp = M(68, 11) * x(11)
      temp = temp + M(68, 65) * x(65)
      x(68) = ( x(68) - temp ) / M(68, 68)

      temp = M(69, 3) * x(3)
      temp = temp + M(69, 12) * x(12)
      temp = temp + M(69, 26) * x(26)
      temp = temp + M(69, 46) * x(46)
      temp = temp + M(69, 48) * x(48)
      temp = temp + M(69, 49) * x(49)
      temp = temp + M(69, 52) * x(52)
      temp = temp + M(69, 53) * x(53)
      temp = temp + M(69, 54) * x(54)
      temp = temp + M(69, 55) * x(55)
      temp = temp + M(69, 56) * x(56)
      temp = temp + M(69, 57) * x(57)
      temp = temp + M(69, 58) * x(58)
      temp = temp + M(69, 59) * x(59)
      temp = temp + M(69, 62) * x(62)
      temp = temp + M(69, 64) * x(64)
      temp = temp + M(69, 65) * x(65)
      temp = temp + M(69, 68) * x(68)
      x(69) = ( x(69) - temp ) / M(69, 69)

      temp = M(70, 14) * x(14)
      temp = temp + M(70, 26) * x(26)
      temp = temp + M(70, 30) * x(30)
      temp = temp + M(70, 48) * x(48)
      temp = temp + M(70, 49) * x(49)
      temp = temp + M(70, 53) * x(53)
      temp = temp + M(70, 56) * x(56)
      temp = temp + M(70, 59) * x(59)
      temp = temp + M(70, 68) * x(68)
      temp = temp + M(70, 69) * x(69)
      x(70) = ( x(70) - temp ) / M(70, 70)

      temp = M(71, 4) * x(4)
      temp = temp + M(71, 11) * x(11)
      temp = temp + M(71, 12) * x(12)
      temp = temp + M(71, 26) * x(26)
      temp = temp + M(71, 34) * x(34)
      temp = temp + M(71, 46) * x(46)
      temp = temp + M(71, 48) * x(48)
      temp = temp + M(71, 49) * x(49)
      temp = temp + M(71, 50) * x(50)
      temp = temp + M(71, 52) * x(52)
      temp = temp + M(71, 53) * x(53)
      temp = temp + M(71, 54) * x(54)
      temp = temp + M(71, 55) * x(55)
      temp = temp + M(71, 56) * x(56)
      temp = temp + M(71, 57) * x(57)
      temp = temp + M(71, 58) * x(58)
      temp = temp + M(71, 59) * x(59)
      temp = temp + M(71, 62) * x(62)
      temp = temp + M(71, 64) * x(64)
      temp = temp + M(71, 65) * x(65)
      temp = temp + M(71, 68) * x(68)
      temp = temp + M(71, 69) * x(69)
      temp = temp + M(71, 70) * x(70)
      x(71) = ( x(71) - temp ) / M(71, 71)

      temp = M(72, 5) * x(5)
      temp = temp + M(72, 6) * x(6)
      temp = temp + M(72, 21) * x(21)
      temp = temp + M(72, 28) * x(28)
      temp = temp + M(72, 34) * x(34)
      temp = temp + M(72, 35) * x(35)
      temp = temp + M(72, 37) * x(37)
      temp = temp + M(72, 42) * x(42)
      temp = temp + M(72, 44) * x(44)
      temp = temp + M(72, 46) * x(46)
      temp = temp + M(72, 48) * x(48)
      temp = temp + M(72, 49) * x(49)
      temp = temp + M(72, 52) * x(52)
      temp = temp + M(72, 59) * x(59)
      temp = temp + M(72, 64) * x(64)
      temp = temp + M(72, 65) * x(65)
      temp = temp + M(72, 68) * x(68)
      temp = temp + M(72, 69) * x(69)
      temp = temp + M(72, 70) * x(70)
      x(72) = ( x(72) - temp ) / M(72, 72)

      temp = M(73, 21) * x(21)
      temp = temp + M(73, 25) * x(25)
      temp = temp + M(73, 28) * x(28)
      temp = temp + M(73, 29) * x(29)
      temp = temp + M(73, 33) * x(33)
      temp = temp + M(73, 34) * x(34)
      temp = temp + M(73, 35) * x(35)
      temp = temp + M(73, 37) * x(37)
      temp = temp + M(73, 39) * x(39)
      temp = temp + M(73, 40) * x(40)
      temp = temp + M(73, 46) * x(46)
      temp = temp + M(73, 47) * x(47)
      temp = temp + M(73, 48) * x(48)
      temp = temp + M(73, 49) * x(49)
      temp = temp + M(73, 50) * x(50)
      temp = temp + M(73, 51) * x(51)
      temp = temp + M(73, 52) * x(52)
      temp = temp + M(73, 53) * x(53)
      temp = temp + M(73, 54) * x(54)
      temp = temp + M(73, 55) * x(55)
      temp = temp + M(73, 56) * x(56)
      temp = temp + M(73, 57) * x(57)
      temp = temp + M(73, 58) * x(58)
      temp = temp + M(73, 59) * x(59)
      temp = temp + M(73, 62) * x(62)
      temp = temp + M(73, 64) * x(64)
      temp = temp + M(73, 68) * x(68)
      temp = temp + M(73, 69) * x(69)
      temp = temp + M(73, 70) * x(70)
      temp = temp + M(73, 71) * x(71)
      temp = temp + M(73, 72) * x(72)
      x(73) = ( x(73) - temp ) / M(73, 73)

      temp = M(74, 11) * x(11)
      temp = temp + M(74, 14) * x(14)
      temp = temp + M(74, 21) * x(21)
      temp = temp + M(74, 25) * x(25)
      temp = temp + M(74, 27) * x(27)
      temp = temp + M(74, 28) * x(28)
      temp = temp + M(74, 33) * x(33)
      temp = temp + M(74, 34) * x(34)
      temp = temp + M(74, 35) * x(35)
      temp = temp + M(74, 37) * x(37)
      temp = temp + M(74, 39) * x(39)
      temp = temp + M(74, 40) * x(40)
      temp = temp + M(74, 41) * x(41)
      temp = temp + M(74, 42) * x(42)
      temp = temp + M(74, 44) * x(44)
      temp = temp + M(74, 46) * x(46)
      temp = temp + M(74, 47) * x(47)
      temp = temp + M(74, 48) * x(48)
      temp = temp + M(74, 49) * x(49)
      temp = temp + M(74, 50) * x(50)
      temp = temp + M(74, 51) * x(51)
      temp = temp + M(74, 52) * x(52)
      temp = temp + M(74, 53) * x(53)
      temp = temp + M(74, 54) * x(54)
      temp = temp + M(74, 55) * x(55)
      temp = temp + M(74, 56) * x(56)
      temp = temp + M(74, 57) * x(57)
      temp = temp + M(74, 58) * x(58)
      temp = temp + M(74, 59) * x(59)
      temp = temp + M(74, 61) * x(61)
      temp = temp + M(74, 62) * x(62)
      temp = temp + M(74, 64) * x(64)
      temp = temp + M(74, 68) * x(68)
      temp = temp + M(74, 69) * x(69)
      temp = temp + M(74, 70) * x(70)
      temp = temp + M(74, 72) * x(72)
      temp = temp + M(74, 73) * x(73)
      x(74) = ( x(74) - temp ) / M(74, 74)

      temp = M(75, 3) * x(3)
      temp = temp + M(75, 4) * x(4)
      temp = temp + M(75, 5) * x(5)
      temp = temp + M(75, 6) * x(6)
      temp = temp + M(75, 7) * x(7)
      temp = temp + M(75, 9) * x(9)
      temp = temp + M(75, 11) * x(11)
      temp = temp + M(75, 12) * x(12)
      temp = temp + M(75, 14) * x(14)
      temp = temp + M(75, 17) * x(17)
      temp = temp + M(75, 18) * x(18)
      temp = temp + M(75, 21) * x(21)
      temp = temp + M(75, 24) * x(24)
      temp = temp + M(75, 25) * x(25)
      temp = temp + M(75, 26) * x(26)
      temp = temp + M(75, 27) * x(27)
      temp = temp + M(75, 28) * x(28)
      temp = temp + M(75, 29) * x(29)
      temp = temp + M(75, 30) * x(30)
      temp = temp + M(75, 31) * x(31)
      temp = temp + M(75, 32) * x(32)
      temp = temp + M(75, 34) * x(34)
      temp = temp + M(75, 35) * x(35)
      temp = temp + M(75, 37) * x(37)
      temp = temp + M(75, 41) * x(41)
      temp = temp + M(75, 42) * x(42)
      temp = temp + M(75, 44) * x(44)
      temp = temp + M(75, 45) * x(45)
      temp = temp + M(75, 46) * x(46)
      temp = temp + M(75, 52) * x(52)
      temp = temp + M(75, 53) * x(53)
      temp = temp + M(75, 56) * x(56)
      temp = temp + M(75, 59) * x(59)
      temp = temp + M(75, 60) * x(60)
      temp = temp + M(75, 61) * x(61)
      temp = temp + M(75, 64) * x(64)
      temp = temp + M(75, 65) * x(65)
      temp = temp + M(75, 66) * x(66)
      temp = temp + M(75, 67) * x(67)
      temp = temp + M(75, 68) * x(68)
      temp = temp + M(75, 69) * x(69)
      temp = temp + M(75, 70) * x(70)
      temp = temp + M(75, 71) * x(71)
      temp = temp + M(75, 72) * x(72)
      temp = temp + M(75, 73) * x(73)
      temp = temp + M(75, 74) * x(74)
      x(75) = ( x(75) - temp ) / M(75, 75)

      temp = M(76, 21) * x(21)
      temp = temp + M(76, 28) * x(28)
      temp = temp + M(76, 33) * x(33)
      temp = temp + M(76, 34) * x(34)
      temp = temp + M(76, 35) * x(35)
      temp = temp + M(76, 37) * x(37)
      temp = temp + M(76, 39) * x(39)
      temp = temp + M(76, 40) * x(40)
      temp = temp + M(76, 41) * x(41)
      temp = temp + M(76, 42) * x(42)
      temp = temp + M(76, 44) * x(44)
      temp = temp + M(76, 46) * x(46)
      temp = temp + M(76, 47) * x(47)
      temp = temp + M(76, 48) * x(48)
      temp = temp + M(76, 49) * x(49)
      temp = temp + M(76, 50) * x(50)
      temp = temp + M(76, 51) * x(51)
      temp = temp + M(76, 52) * x(52)
      temp = temp + M(76, 54) * x(54)
      temp = temp + M(76, 55) * x(55)
      temp = temp + M(76, 57) * x(57)
      temp = temp + M(76, 58) * x(58)
      temp = temp + M(76, 59) * x(59)
      temp = temp + M(76, 62) * x(62)
      temp = temp + M(76, 64) * x(64)
      temp = temp + M(76, 66) * x(66)
      temp = temp + M(76, 68) * x(68)
      temp = temp + M(76, 69) * x(69)
      temp = temp + M(76, 70) * x(70)
      temp = temp + M(76, 71) * x(71)
      temp = temp + M(76, 72) * x(72)
      temp = temp + M(76, 73) * x(73)
      temp = temp + M(76, 75) * x(75)
      x(76) = ( x(76) - temp ) / M(76, 76)

      temp = M(77, 13) * x(13)
      temp = temp + M(77, 14) * x(14)
      temp = temp + M(77, 21) * x(21)
      temp = temp + M(77, 24) * x(24)
      temp = temp + M(77, 26) * x(26)
      temp = temp + M(77, 27) * x(27)
      temp = temp + M(77, 28) * x(28)
      temp = temp + M(77, 33) * x(33)
      temp = temp + M(77, 34) * x(34)
      temp = temp + M(77, 37) * x(37)
      temp = temp + M(77, 39) * x(39)
      temp = temp + M(77, 40) * x(40)
      temp = temp + M(77, 42) * x(42)
      temp = temp + M(77, 44) * x(44)
      temp = temp + M(77, 46) * x(46)
      temp = temp + M(77, 47) * x(47)
      temp = temp + M(77, 48) * x(48)
      temp = temp + M(77, 49) * x(49)
      temp = temp + M(77, 50) * x(50)
      temp = temp + M(77, 51) * x(51)
      temp = temp + M(77, 52) * x(52)
      temp = temp + M(77, 53) * x(53)
      temp = temp + M(77, 54) * x(54)
      temp = temp + M(77, 55) * x(55)
      temp = temp + M(77, 56) * x(56)
      temp = temp + M(77, 57) * x(57)
      temp = temp + M(77, 58) * x(58)
      temp = temp + M(77, 59) * x(59)
      temp = temp + M(77, 61) * x(61)
      temp = temp + M(77, 62) * x(62)
      temp = temp + M(77, 64) * x(64)
      temp = temp + M(77, 66) * x(66)
      temp = temp + M(77, 67) * x(67)
      temp = temp + M(77, 68) * x(68)
      temp = temp + M(77, 69) * x(69)
      temp = temp + M(77, 70) * x(70)
      temp = temp + M(77, 71) * x(71)
      temp = temp + M(77, 72) * x(72)
      temp = temp + M(77, 73) * x(73)
      temp = temp + M(77, 74) * x(74)
      temp = temp + M(77, 75) * x(75)
      temp = temp + M(77, 76) * x(76)
      x(77) = ( x(77) - temp ) / M(77, 77)

      temp = M(78, 32) * x(32)
      temp = temp + M(78, 33) * x(33)
      temp = temp + M(78, 34) * x(34)
      temp = temp + M(78, 37) * x(37)
      temp = temp + M(78, 39) * x(39)
      temp = temp + M(78, 40) * x(40)
      temp = temp + M(78, 47) * x(47)
      temp = temp + M(78, 48) * x(48)
      temp = temp + M(78, 49) * x(49)
      temp = temp + M(78, 50) * x(50)
      temp = temp + M(78, 51) * x(51)
      temp = temp + M(78, 52) * x(52)
      temp = temp + M(78, 54) * x(54)
      temp = temp + M(78, 55) * x(55)
      temp = temp + M(78, 57) * x(57)
      temp = temp + M(78, 58) * x(58)
      temp = temp + M(78, 59) * x(59)
      temp = temp + M(78, 62) * x(62)
      temp = temp + M(78, 64) * x(64)
      temp = temp + M(78, 68) * x(68)
      temp = temp + M(78, 69) * x(69)
      temp = temp + M(78, 70) * x(70)
      temp = temp + M(78, 72) * x(72)
      temp = temp + M(78, 73) * x(73)
      temp = temp + M(78, 75) * x(75)
      temp = temp + M(78, 76) * x(76)
      temp = temp + M(78, 77) * x(77)
      x(78) = ( x(78) - temp ) / M(78, 78)

      temp = M(79, 42) * x(42)
      temp = temp + M(79, 46) * x(46)
      temp = temp + M(79, 52) * x(52)
      temp = temp + M(79, 59) * x(59)
      temp = temp + M(79, 73) * x(73)
      temp = temp + M(79, 75) * x(75)
      temp = temp + M(79, 76) * x(76)
      temp = temp + M(79, 77) * x(77)
      temp = temp + M(79, 78) * x(78)
      x(79) = ( x(79) - temp ) / M(79, 79)

      temp = M(80, 13) * x(13)
      temp = temp + M(80, 17) * x(17)
      temp = temp + M(80, 18) * x(18)
      temp = temp + M(80, 24) * x(24)
      temp = temp + M(80, 31) * x(31)
      temp = temp + M(80, 32) * x(32)
      temp = temp + M(80, 33) * x(33)
      temp = temp + M(80, 34) * x(34)
      temp = temp + M(80, 36) * x(36)
      temp = temp + M(80, 37) * x(37)
      temp = temp + M(80, 39) * x(39)
      temp = temp + M(80, 40) * x(40)
      temp = temp + M(80, 42) * x(42)
      temp = temp + M(80, 44) * x(44)
      temp = temp + M(80, 46) * x(46)
      temp = temp + M(80, 47) * x(47)
      temp = temp + M(80, 48) * x(48)
      temp = temp + M(80, 49) * x(49)
      temp = temp + M(80, 50) * x(50)
      temp = temp + M(80, 51) * x(51)
      temp = temp + M(80, 52) * x(52)
      temp = temp + M(80, 54) * x(54)
      temp = temp + M(80, 55) * x(55)
      temp = temp + M(80, 57) * x(57)
      temp = temp + M(80, 58) * x(58)
      temp = temp + M(80, 59) * x(59)
      temp = temp + M(80, 62) * x(62)
      temp = temp + M(80, 64) * x(64)
      temp = temp + M(80, 67) * x(67)
      temp = temp + M(80, 68) * x(68)
      temp = temp + M(80, 69) * x(69)
      temp = temp + M(80, 70) * x(70)
      temp = temp + M(80, 71) * x(71)
      temp = temp + M(80, 72) * x(72)
      temp = temp + M(80, 73) * x(73)
      temp = temp + M(80, 74) * x(74)
      temp = temp + M(80, 75) * x(75)
      temp = temp + M(80, 76) * x(76)
      temp = temp + M(80, 77) * x(77)
      temp = temp + M(80, 78) * x(78)
      temp = temp + M(80, 79) * x(79)
      x(80) = ( x(80) - temp ) / M(80, 80)

      temp = M(81, 3) * x(3)
      temp = temp + M(81, 4) * x(4)
      temp = temp + M(81, 5) * x(5)
      temp = temp + M(81, 6) * x(6)
      temp = temp + M(81, 9) * x(9)
      temp = temp + M(81, 11) * x(11)
      temp = temp + M(81, 12) * x(12)
      temp = temp + M(81, 14) * x(14)
      temp = temp + M(81, 17) * x(17)
      temp = temp + M(81, 18) * x(18)
      temp = temp + M(81, 21) * x(21)
      temp = temp + M(81, 24) * x(24)
      temp = temp + M(81, 25) * x(25)
      temp = temp + M(81, 26) * x(26)
      temp = temp + M(81, 27) * x(27)
      temp = temp + M(81, 28) * x(28)
      temp = temp + M(81, 31) * x(31)
      temp = temp + M(81, 33) * x(33)
      temp = temp + M(81, 34) * x(34)
      temp = temp + M(81, 35) * x(35)
      temp = temp + M(81, 36) * x(36)
      temp = temp + M(81, 37) * x(37)
      temp = temp + M(81, 39) * x(39)
      temp = temp + M(81, 40) * x(40)
      temp = temp + M(81, 41) * x(41)
      temp = temp + M(81, 42) * x(42)
      temp = temp + M(81, 45) * x(45)
      temp = temp + M(81, 46) * x(46)
      temp = temp + M(81, 47) * x(47)
      temp = temp + M(81, 48) * x(48)
      temp = temp + M(81, 49) * x(49)
      temp = temp + M(81, 50) * x(50)
      temp = temp + M(81, 51) * x(51)
      temp = temp + M(81, 52) * x(52)
      temp = temp + M(81, 53) * x(53)
      temp = temp + M(81, 54) * x(54)
      temp = temp + M(81, 55) * x(55)
      temp = temp + M(81, 56) * x(56)
      temp = temp + M(81, 57) * x(57)
      temp = temp + M(81, 58) * x(58)
      temp = temp + M(81, 59) * x(59)
      temp = temp + M(81, 60) * x(60)
      temp = temp + M(81, 61) * x(61)
      temp = temp + M(81, 62) * x(62)
      temp = temp + M(81, 64) * x(64)
      temp = temp + M(81, 65) * x(65)
      temp = temp + M(81, 66) * x(66)
      temp = temp + M(81, 68) * x(68)
      temp = temp + M(81, 69) * x(69)
      temp = temp + M(81, 70) * x(70)
      temp = temp + M(81, 71) * x(71)
      temp = temp + M(81, 72) * x(72)
      temp = temp + M(81, 73) * x(73)
      temp = temp + M(81, 74) * x(74)
      temp = temp + M(81, 75) * x(75)
      temp = temp + M(81, 76) * x(76)
      temp = temp + M(81, 77) * x(77)
      temp = temp + M(81, 78) * x(78)
      temp = temp + M(81, 79) * x(79)
      temp = temp + M(81, 80) * x(80)
      x(81) = ( x(81) - temp ) / M(81, 81)

      temp = M(82, 36) * x(36)
      temp = temp + M(82, 37) * x(37)
      temp = temp + M(82, 39) * x(39)
      temp = temp + M(82, 40) * x(40)
      temp = temp + M(82, 42) * x(42)
      temp = temp + M(82, 47) * x(47)
      temp = temp + M(82, 48) * x(48)
      temp = temp + M(82, 49) * x(49)
      temp = temp + M(82, 51) * x(51)
      temp = temp + M(82, 52) * x(52)
      temp = temp + M(82, 55) * x(55)
      temp = temp + M(82, 57) * x(57)
      temp = temp + M(82, 58) * x(58)
      temp = temp + M(82, 59) * x(59)
      temp = temp + M(82, 68) * x(68)
      temp = temp + M(82, 69) * x(69)
      temp = temp + M(82, 70) * x(70)
      temp = temp + M(82, 72) * x(72)
      temp = temp + M(82, 73) * x(73)
      temp = temp + M(82, 75) * x(75)
      temp = temp + M(82, 76) * x(76)
      temp = temp + M(82, 77) * x(77)
      temp = temp + M(82, 78) * x(78)
      temp = temp + M(82, 79) * x(79)
      temp = temp + M(82, 80) * x(80)
      temp = temp + M(82, 81) * x(81)
      x(82) = ( x(82) - temp ) / M(82, 82)


C     Backward substitution.

      temp = M(81, 82) * x(82)
      x(81) = x(81) - temp

      temp = M(80, 81) * x(81)
      temp = temp + M(80, 82) * x(82)
      x(80) = x(80) - temp

      temp = M(79, 80) * x(80)
      temp = temp + M(79, 81) * x(81)
      temp = temp + M(79, 82) * x(82)
      x(79) = x(79) - temp

      temp = M(78, 79) * x(79)
      temp = temp + M(78, 80) * x(80)
      temp = temp + M(78, 81) * x(81)
      temp = temp + M(78, 82) * x(82)
      x(78) = x(78) - temp

      temp = M(77, 78) * x(78)
      temp = temp + M(77, 79) * x(79)
      temp = temp + M(77, 80) * x(80)
      temp = temp + M(77, 81) * x(81)
      temp = temp + M(77, 82) * x(82)
      x(77) = x(77) - temp

      temp = M(76, 77) * x(77)
      temp = temp + M(76, 78) * x(78)
      temp = temp + M(76, 79) * x(79)
      temp = temp + M(76, 80) * x(80)
      temp = temp + M(76, 81) * x(81)
      temp = temp + M(76, 82) * x(82)
      x(76) = x(76) - temp

      temp = M(75, 76) * x(76)
      temp = temp + M(75, 77) * x(77)
      temp = temp + M(75, 78) * x(78)
      temp = temp + M(75, 79) * x(79)
      temp = temp + M(75, 80) * x(80)
      temp = temp + M(75, 81) * x(81)
      temp = temp + M(75, 82) * x(82)
      x(75) = x(75) - temp

      temp = M(74, 75) * x(75)
      temp = temp + M(74, 76) * x(76)
      temp = temp + M(74, 77) * x(77)
      temp = temp + M(74, 78) * x(78)
      temp = temp + M(74, 79) * x(79)
      temp = temp + M(74, 80) * x(80)
      temp = temp + M(74, 81) * x(81)
      temp = temp + M(74, 82) * x(82)
      x(74) = x(74) - temp

      temp = M(73, 75) * x(75)
      temp = temp + M(73, 76) * x(76)
      temp = temp + M(73, 77) * x(77)
      temp = temp + M(73, 78) * x(78)
      temp = temp + M(73, 79) * x(79)
      temp = temp + M(73, 80) * x(80)
      temp = temp + M(73, 81) * x(81)
      temp = temp + M(73, 82) * x(82)
      x(73) = x(73) - temp

      temp = M(72, 73) * x(73)
      temp = temp + M(72, 75) * x(75)
      temp = temp + M(72, 76) * x(76)
      temp = temp + M(72, 77) * x(77)
      temp = temp + M(72, 78) * x(78)
      temp = temp + M(72, 79) * x(79)
      temp = temp + M(72, 80) * x(80)
      temp = temp + M(72, 81) * x(81)
      temp = temp + M(72, 82) * x(82)
      x(72) = x(72) - temp

      temp = M(71, 72) * x(72)
      temp = temp + M(71, 73) * x(73)
      temp = temp + M(71, 75) * x(75)
      temp = temp + M(71, 76) * x(76)
      temp = temp + M(71, 77) * x(77)
      temp = temp + M(71, 78) * x(78)
      temp = temp + M(71, 79) * x(79)
      temp = temp + M(71, 80) * x(80)
      temp = temp + M(71, 81) * x(81)
      temp = temp + M(71, 82) * x(82)
      x(71) = x(71) - temp

      temp = M(70, 72) * x(72)
      temp = temp + M(70, 73) * x(73)
      temp = temp + M(70, 75) * x(75)
      temp = temp + M(70, 76) * x(76)
      temp = temp + M(70, 77) * x(77)
      temp = temp + M(70, 78) * x(78)
      temp = temp + M(70, 79) * x(79)
      temp = temp + M(70, 80) * x(80)
      temp = temp + M(70, 81) * x(81)
      temp = temp + M(70, 82) * x(82)
      x(70) = x(70) - temp

      temp = M(69, 70) * x(70)
      temp = temp + M(69, 72) * x(72)
      temp = temp + M(69, 73) * x(73)
      temp = temp + M(69, 75) * x(75)
      temp = temp + M(69, 76) * x(76)
      temp = temp + M(69, 77) * x(77)
      temp = temp + M(69, 78) * x(78)
      temp = temp + M(69, 79) * x(79)
      temp = temp + M(69, 80) * x(80)
      temp = temp + M(69, 81) * x(81)
      temp = temp + M(69, 82) * x(82)
      x(69) = x(69) - temp

      temp = M(68, 69) * x(69)
      temp = temp + M(68, 70) * x(70)
      temp = temp + M(68, 72) * x(72)
      temp = temp + M(68, 73) * x(73)
      temp = temp + M(68, 75) * x(75)
      temp = temp + M(68, 76) * x(76)
      temp = temp + M(68, 77) * x(77)
      temp = temp + M(68, 78) * x(78)
      temp = temp + M(68, 79) * x(79)
      temp = temp + M(68, 80) * x(80)
      temp = temp + M(68, 81) * x(81)
      temp = temp + M(68, 82) * x(82)
      x(68) = x(68) - temp

      temp = M(67, 68) * x(68)
      temp = temp + M(67, 69) * x(69)
      temp = temp + M(67, 71) * x(71)
      temp = temp + M(67, 73) * x(73)
      temp = temp + M(67, 74) * x(74)
      temp = temp + M(67, 75) * x(75)
      temp = temp + M(67, 76) * x(76)
      temp = temp + M(67, 77) * x(77)
      temp = temp + M(67, 78) * x(78)
      temp = temp + M(67, 79) * x(79)
      temp = temp + M(67, 80) * x(80)
      temp = temp + M(67, 81) * x(81)
      x(67) = x(67) - temp

      temp = M(66, 68) * x(68)
      temp = temp + M(66, 69) * x(69)
      temp = temp + M(66, 73) * x(73)
      temp = temp + M(66, 75) * x(75)
      temp = temp + M(66, 76) * x(76)
      temp = temp + M(66, 77) * x(77)
      temp = temp + M(66, 78) * x(78)
      temp = temp + M(66, 79) * x(79)
      temp = temp + M(66, 80) * x(80)
      temp = temp + M(66, 81) * x(81)
      x(66) = x(66) - temp

      temp = M(65, 68) * x(68)
      temp = temp + M(65, 69) * x(69)
      temp = temp + M(65, 70) * x(70)
      temp = temp + M(65, 72) * x(72)
      temp = temp + M(65, 73) * x(73)
      temp = temp + M(65, 75) * x(75)
      temp = temp + M(65, 76) * x(76)
      temp = temp + M(65, 77) * x(77)
      temp = temp + M(65, 78) * x(78)
      temp = temp + M(65, 79) * x(79)
      temp = temp + M(65, 80) * x(80)
      temp = temp + M(65, 81) * x(81)
      x(65) = x(65) - temp

      temp = M(64, 69) * x(69)
      temp = temp + M(64, 73) * x(73)
      temp = temp + M(64, 75) * x(75)
      temp = temp + M(64, 76) * x(76)
      temp = temp + M(64, 77) * x(77)
      temp = temp + M(64, 78) * x(78)
      temp = temp + M(64, 79) * x(79)
      temp = temp + M(64, 80) * x(80)
      temp = temp + M(64, 81) * x(81)
      x(64) = x(64) - temp

      temp = M(63, 64) * x(64)
      temp = temp + M(63, 68) * x(68)
      temp = temp + M(63, 70) * x(70)
      temp = temp + M(63, 73) * x(73)
      temp = temp + M(63, 75) * x(75)
      temp = temp + M(63, 76) * x(76)
      temp = temp + M(63, 77) * x(77)
      temp = temp + M(63, 78) * x(78)
      temp = temp + M(63, 79) * x(79)
      temp = temp + M(63, 80) * x(80)
      temp = temp + M(63, 81) * x(81)
      x(63) = x(63) - temp

      temp = M(62, 73) * x(73)
      temp = temp + M(62, 75) * x(75)
      temp = temp + M(62, 76) * x(76)
      temp = temp + M(62, 77) * x(77)
      temp = temp + M(62, 78) * x(78)
      temp = temp + M(62, 79) * x(79)
      temp = temp + M(62, 80) * x(80)
      temp = temp + M(62, 81) * x(81)
      x(62) = x(62) - temp

      temp = M(61, 68) * x(68)
      temp = temp + M(61, 73) * x(73)
      temp = temp + M(61, 75) * x(75)
      temp = temp + M(61, 76) * x(76)
      temp = temp + M(61, 77) * x(77)
      temp = temp + M(61, 78) * x(78)
      temp = temp + M(61, 79) * x(79)
      temp = temp + M(61, 80) * x(80)
      temp = temp + M(61, 81) * x(81)
      x(61) = x(61) - temp

      temp = M(60, 61) * x(61)
      temp = temp + M(60, 66) * x(66)
      temp = temp + M(60, 71) * x(71)
      temp = temp + M(60, 73) * x(73)
      temp = temp + M(60, 74) * x(74)
      temp = temp + M(60, 75) * x(75)
      temp = temp + M(60, 76) * x(76)
      temp = temp + M(60, 77) * x(77)
      temp = temp + M(60, 78) * x(78)
      temp = temp + M(60, 79) * x(79)
      temp = temp + M(60, 80) * x(80)
      temp = temp + M(60, 81) * x(81)
      x(60) = x(60) - temp

      temp = M(59, 73) * x(73)
      temp = temp + M(59, 75) * x(75)
      temp = temp + M(59, 76) * x(76)
      temp = temp + M(59, 77) * x(77)
      temp = temp + M(59, 78) * x(78)
      temp = temp + M(59, 79) * x(79)
      temp = temp + M(59, 80) * x(80)
      temp = temp + M(59, 81) * x(81)
      x(59) = x(59) - temp

      temp = M(58, 59) * x(59)
      temp = temp + M(58, 73) * x(73)
      temp = temp + M(58, 75) * x(75)
      temp = temp + M(58, 76) * x(76)
      temp = temp + M(58, 77) * x(77)
      temp = temp + M(58, 78) * x(78)
      temp = temp + M(58, 80) * x(80)
      temp = temp + M(58, 81) * x(81)
      x(58) = x(58) - temp

      temp = M(57, 58) * x(58)
      temp = temp + M(57, 59) * x(59)
      temp = temp + M(57, 73) * x(73)
      temp = temp + M(57, 75) * x(75)
      temp = temp + M(57, 76) * x(76)
      temp = temp + M(57, 77) * x(77)
      temp = temp + M(57, 78) * x(78)
      temp = temp + M(57, 80) * x(80)
      temp = temp + M(57, 81) * x(81)
      x(57) = x(57) - temp

      temp = M(56, 59) * x(59)
      temp = temp + M(56, 73) * x(73)
      temp = temp + M(56, 75) * x(75)
      temp = temp + M(56, 76) * x(76)
      temp = temp + M(56, 77) * x(77)
      temp = temp + M(56, 78) * x(78)
      temp = temp + M(56, 80) * x(80)
      temp = temp + M(56, 81) * x(81)
      x(56) = x(56) - temp

      temp = M(55, 59) * x(59)
      temp = temp + M(55, 73) * x(73)
      temp = temp + M(55, 75) * x(75)
      temp = temp + M(55, 76) * x(76)
      temp = temp + M(55, 77) * x(77)
      temp = temp + M(55, 78) * x(78)
      temp = temp + M(55, 81) * x(81)
      x(55) = x(55) - temp

      temp = M(54, 59) * x(59)
      temp = temp + M(54, 73) * x(73)
      temp = temp + M(54, 75) * x(75)
      temp = temp + M(54, 76) * x(76)
      temp = temp + M(54, 77) * x(77)
      temp = temp + M(54, 78) * x(78)
      temp = temp + M(54, 80) * x(80)
      temp = temp + M(54, 81) * x(81)
      x(54) = x(54) - temp

      temp = M(53, 59) * x(59)
      temp = temp + M(53, 73) * x(73)
      temp = temp + M(53, 75) * x(75)
      temp = temp + M(53, 76) * x(76)
      temp = temp + M(53, 77) * x(77)
      temp = temp + M(53, 78) * x(78)
      temp = temp + M(53, 80) * x(80)
      temp = temp + M(53, 81) * x(81)
      x(53) = x(53) - temp

      temp = M(52, 59) * x(59)
      temp = temp + M(52, 73) * x(73)
      temp = temp + M(52, 75) * x(75)
      temp = temp + M(52, 76) * x(76)
      temp = temp + M(52, 77) * x(77)
      temp = temp + M(52, 78) * x(78)
      temp = temp + M(52, 80) * x(80)
      temp = temp + M(52, 81) * x(81)
      x(52) = x(52) - temp

      temp = M(51, 52) * x(52)
      temp = temp + M(51, 59) * x(59)
      temp = temp + M(51, 73) * x(73)
      temp = temp + M(51, 75) * x(75)
      temp = temp + M(51, 76) * x(76)
      temp = temp + M(51, 77) * x(77)
      temp = temp + M(51, 78) * x(78)
      temp = temp + M(51, 80) * x(80)
      temp = temp + M(51, 81) * x(81)
      x(51) = x(51) - temp

      temp = M(50, 59) * x(59)
      temp = temp + M(50, 73) * x(73)
      temp = temp + M(50, 75) * x(75)
      temp = temp + M(50, 76) * x(76)
      temp = temp + M(50, 77) * x(77)
      temp = temp + M(50, 78) * x(78)
      temp = temp + M(50, 81) * x(81)
      x(50) = x(50) - temp

      temp = M(49, 73) * x(73)
      temp = temp + M(49, 75) * x(75)
      temp = temp + M(49, 76) * x(76)
      temp = temp + M(49, 77) * x(77)
      temp = temp + M(49, 78) * x(78)
      temp = temp + M(49, 81) * x(81)
      x(49) = x(49) - temp

      temp = M(48, 73) * x(73)
      temp = temp + M(48, 75) * x(75)
      temp = temp + M(48, 76) * x(76)
      temp = temp + M(48, 77) * x(77)
      temp = temp + M(48, 78) * x(78)
      temp = temp + M(48, 81) * x(81)
      x(48) = x(48) - temp

      temp = M(47, 59) * x(59)
      temp = temp + M(47, 73) * x(73)
      temp = temp + M(47, 75) * x(75)
      temp = temp + M(47, 76) * x(76)
      temp = temp + M(47, 77) * x(77)
      temp = temp + M(47, 78) * x(78)
      temp = temp + M(47, 81) * x(81)
      x(47) = x(47) - temp

      temp = M(46, 52) * x(52)
      temp = temp + M(46, 59) * x(59)
      temp = temp + M(46, 73) * x(73)
      temp = temp + M(46, 75) * x(75)
      temp = temp + M(46, 76) * x(76)
      temp = temp + M(46, 77) * x(77)
      temp = temp + M(46, 78) * x(78)
      temp = temp + M(46, 80) * x(80)
      temp = temp + M(46, 81) * x(81)
      x(46) = x(46) - temp

      temp = M(45, 52) * x(52)
      temp = temp + M(45, 53) * x(53)
      temp = temp + M(45, 56) * x(56)
      temp = temp + M(45, 59) * x(59)
      temp = temp + M(45, 75) * x(75)
      temp = temp + M(45, 77) * x(77)
      temp = temp + M(45, 78) * x(78)
      temp = temp + M(45, 80) * x(80)
      temp = temp + M(45, 81) * x(81)
      x(45) = x(45) - temp

      temp = M(44, 59) * x(59)
      temp = temp + M(44, 75) * x(75)
      temp = temp + M(44, 76) * x(76)
      temp = temp + M(44, 77) * x(77)
      temp = temp + M(44, 79) * x(79)
      temp = temp + M(44, 80) * x(80)
      x(44) = x(44) - temp

      temp = M(43, 46) * x(46)
      temp = temp + M(43, 52) * x(52)
      temp = temp + M(43, 53) * x(53)
      temp = temp + M(43, 59) * x(59)
      temp = temp + M(43, 75) * x(75)
      temp = temp + M(43, 77) * x(77)
      temp = temp + M(43, 78) * x(78)
      temp = temp + M(43, 79) * x(79)
      temp = temp + M(43, 80) * x(80)
      x(43) = x(43) - temp

      temp = M(42, 59) * x(59)
      temp = temp + M(42, 75) * x(75)
      temp = temp + M(42, 77) * x(77)
      temp = temp + M(42, 79) * x(79)
      temp = temp + M(42, 80) * x(80)
      x(42) = x(42) - temp

      temp = M(41, 42) * x(42)
      temp = temp + M(41, 52) * x(52)
      temp = temp + M(41, 64) * x(64)
      temp = temp + M(41, 73) * x(73)
      temp = temp + M(41, 75) * x(75)
      x(41) = x(41) - temp

      temp = M(40, 59) * x(59)
      temp = temp + M(40, 73) * x(73)
      temp = temp + M(40, 75) * x(75)
      temp = temp + M(40, 76) * x(76)
      temp = temp + M(40, 77) * x(77)
      temp = temp + M(40, 78) * x(78)
      temp = temp + M(40, 80) * x(80)
      temp = temp + M(40, 81) * x(81)
      x(40) = x(40) - temp

      temp = M(39, 59) * x(59)
      temp = temp + M(39, 73) * x(73)
      temp = temp + M(39, 75) * x(75)
      temp = temp + M(39, 76) * x(76)
      temp = temp + M(39, 77) * x(77)
      temp = temp + M(39, 78) * x(78)
      temp = temp + M(39, 80) * x(80)
      temp = temp + M(39, 81) * x(81)
      x(39) = x(39) - temp

      temp = M(38, 53) * x(53)
      temp = temp + M(38, 56) * x(56)
      temp = temp + M(38, 59) * x(59)
      temp = temp + M(38, 75) * x(75)
      temp = temp + M(38, 77) * x(77)
      x(38) = x(38) - temp

      temp = M(37, 59) * x(59)
      temp = temp + M(37, 75) * x(75)
      temp = temp + M(37, 77) * x(77)
      temp = temp + M(37, 80) * x(80)
      temp = temp + M(37, 81) * x(81)
      x(37) = x(37) - temp

      temp = M(36, 37) * x(37)
      temp = temp + M(36, 75) * x(75)
      temp = temp + M(36, 77) * x(77)
      temp = temp + M(36, 80) * x(80)
      temp = temp + M(36, 81) * x(81)
      x(36) = x(36) - temp

      temp = M(35, 46) * x(46)
      temp = temp + M(35, 59) * x(59)
      temp = temp + M(35, 75) * x(75)
      temp = temp + M(35, 76) * x(76)
      temp = temp + M(35, 81) * x(81)
      x(35) = x(35) - temp

      temp = M(34, 52) * x(52)
      temp = temp + M(34, 59) * x(59)
      temp = temp + M(34, 75) * x(75)
      temp = temp + M(34, 77) * x(77)
      temp = temp + M(34, 78) * x(78)
      temp = temp + M(34, 80) * x(80)
      x(34) = x(34) - temp

      temp = M(33, 37) * x(37)
      temp = temp + M(33, 59) * x(59)
      temp = temp + M(33, 73) * x(73)
      temp = temp + M(33, 75) * x(75)
      temp = temp + M(33, 76) * x(76)
      temp = temp + M(33, 77) * x(77)
      temp = temp + M(33, 78) * x(78)
      temp = temp + M(33, 80) * x(80)
      temp = temp + M(33, 81) * x(81)
      x(33) = x(33) - temp

      temp = M(32, 37) * x(37)
      temp = temp + M(32, 59) * x(59)
      temp = temp + M(32, 75) * x(75)
      temp = temp + M(32, 78) * x(78)
      temp = temp + M(32, 80) * x(80)
      x(32) = x(32) - temp

      temp = M(31, 37) * x(37)
      temp = temp + M(31, 59) * x(59)
      temp = temp + M(31, 75) * x(75)
      temp = temp + M(31, 80) * x(80)
      x(31) = x(31) - temp

      temp = M(30, 53) * x(53)
      temp = temp + M(30, 56) * x(56)
      temp = temp + M(30, 59) * x(59)
      temp = temp + M(30, 75) * x(75)
      x(30) = x(30) - temp

      temp = M(29, 53) * x(53)
      temp = temp + M(29, 56) * x(56)
      temp = temp + M(29, 59) * x(59)
      temp = temp + M(29, 75) * x(75)
      x(29) = x(29) - temp

      temp = M(28, 34) * x(34)
      temp = temp + M(28, 59) * x(59)
      temp = temp + M(28, 75) * x(75)
      temp = temp + M(28, 77) * x(77)
      x(28) = x(28) - temp

      temp = M(27, 59) * x(59)
      temp = temp + M(27, 75) * x(75)
      temp = temp + M(27, 77) * x(77)
      x(27) = x(27) - temp

      temp = M(26, 59) * x(59)
      temp = temp + M(26, 75) * x(75)
      temp = temp + M(26, 77) * x(77)
      x(26) = x(26) - temp

      temp = M(25, 73) * x(73)
      temp = temp + M(25, 75) * x(75)
      temp = temp + M(25, 81) * x(81)
      x(25) = x(25) - temp

      temp = M(24, 75) * x(75)
      temp = temp + M(24, 80) * x(80)
      temp = temp + M(24, 81) * x(81)
      x(24) = x(24) - temp

      temp = M(23, 26) * x(26)
      temp = temp + M(23, 59) * x(59)
      temp = temp + M(23, 75) * x(75)
      x(23) = x(23) - temp

      temp = M(22, 26) * x(26)
      temp = temp + M(22, 59) * x(59)
      temp = temp + M(22, 75) * x(75)
      x(22) = x(22) - temp

      temp = M(21, 59) * x(59)
      temp = temp + M(21, 75) * x(75)
      temp = temp + M(21, 77) * x(77)
      x(21) = x(21) - temp

      temp = M(20, 37) * x(37)
      temp = temp + M(20, 75) * x(75)
      temp = temp + M(20, 77) * x(77)
      x(20) = x(20) - temp

      temp = M(19, 37) * x(37)
      temp = temp + M(19, 75) * x(75)
      temp = temp + M(19, 77) * x(77)
      x(19) = x(19) - temp

      temp = M(18, 59) * x(59)
      temp = temp + M(18, 75) * x(75)
      temp = temp + M(18, 80) * x(80)
      x(18) = x(18) - temp

      temp = M(17, 59) * x(59)
      temp = temp + M(17, 75) * x(75)
      temp = temp + M(17, 80) * x(80)
      x(17) = x(17) - temp

      temp = M(16, 59) * x(59)
      temp = temp + M(16, 75) * x(75)
      temp = temp + M(16, 77) * x(77)
      x(16) = x(16) - temp

      temp = M(15, 59) * x(59)
      temp = temp + M(15, 75) * x(75)
      temp = temp + M(15, 77) * x(77)
      x(15) = x(15) - temp

      temp = M(14, 59) * x(59)
      temp = temp + M(14, 75) * x(75)
      temp = temp + M(14, 77) * x(77)
      x(14) = x(14) - temp

      temp = M(13, 77) * x(77)
      temp = temp + M(13, 80) * x(80)
      x(13) = x(13) - temp

      temp = M(12, 46) * x(46)
      temp = temp + M(12, 75) * x(75)
      x(12) = x(12) - temp

      temp = M(11, 75) * x(75)
      x(11) = x(11) - temp

      temp = M(10, 75) * x(75)
      x(10) = x(10) - temp

      temp = M(9, 75) * x(75)
      x(9) = x(9) - temp

      temp = M(8, 75) * x(75)
      x(8) = x(8) - temp

      temp = M(7, 59) * x(59)
      x(7) = x(7) - temp

      temp = M(6, 75) * x(75)
      x(6) = x(6) - temp

      temp = M(5, 75) * x(75)
      x(5) = x(5) - temp

      temp = M(4, 75) * x(75)
      x(4) = x(4) - temp

      temp = M(3, 75) * x(75)
      x(3) = x(3) - temp


      END
