C------------------------------------------------------------------------
C     
C     Copyright (C) 2003-2006 CEREA (ENPC)
C     
C     This file is part of Polair.
C     Polair is a 3D eulerian chemistry-transport-model.
C     
C     Polair is free software; you can redistribute it and/or modify
C     it under the terms of the GNU General Public License as published by
C     the Free Software Foundation; either version 2 of the License, or
C     (at your option) any later version.
C     
C     Polair is distributed in the hope that it will be useful,
C     but WITHOUT ANY WARRANTY; without even the implied warranty of
C     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C     GNU General Public License (file "LICENSE") for more details.
C     
C------------------------------------------------------------------------

      SUBROUTINE CALCLOSS(nbin_aer,qnum,kercg,loss)

C------------------------------------------------------------------------
C     
C     -- DESCRIPTION 
C     
C     This routine computes the coagulation loss coefficient for the GDE.
C     If dc/dt=P(c)-L(c)c, this computes L(c).
C     
C------------------------------------------------------------------------
C     
C     -- INPUT VARIABLES
C     
C     QNUM: aerosol number density ([#part.m^-3]).     
C     
C     -- INPUT/OUTPUT VARIABLES
C     
C     
C     -- OUTPUT VARIABLES
C     
C     LOSS: coagulation loss coefficient ([s^-1]).
C     
C------------------------------------------------------------------------
C     
C     -- REMARKS
C     
C------------------------------------------------------------------------
C     
C     -- MODIFICATIONS
C     
C     2005/3/23: cleaning (Bruno Sportisse, CEREA).
C     
C------------------------------------------------------------------------
C     
C     -- AUTHOR(S)
C     
C     2004: Edouard Debry, CEREA.
C     
C------------------------------------------------------------------------

      IMPLICIT NONE

      INCLUDE 'param.inc'

      INTEGER nbin_aer
      DOUBLE PRECISION qnum(nbin_aer),loss(nbin_aer)
      DOUBLE PRECISION kercg(nbin_aer,nbin_aer)

      INTEGER js,jk
      DOUBLE PRECISION lsum

C     ******zero init
      DO js=1,nbin_aer
         loss(js)=0.D0
      END DO

C     ******compute loss coefficient
      DO js=1,nbin_aer
         lsum=0.D0
         
         DO jk=1,nbin_aer
            lsum=lsum+KERCG(js,jk)*qnum(jk)
         END DO

         loss(js)=lsum
      END DO

      END

