C------------------------------------------------------------------------
C     
C     Copyright (C) 2003-2006 CEREA (ENPC)
C     
C     This file is part of Polair.
C     Polair is a 3D eulerian chemistry-transport-model.
C     
C     Polair is free software; you can redistribute it and/or modify
C     it under the terms of the GNU General Public License as published by
C     the Free Software Foundation; either version 2 of the License, or
C     (at your option) any later version.
C     
C     Polair is distributed in the hope that it will be useful,
C     but WITHOUT ANY WARRANTY; without even the implied warranty of
C     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C     GNU General Public License (file "LICENSE") for more details.
C     
C------------------------------------------------------------------------

      SUBROUTINE EQPART(nbin_aer,is1,is2,neq,q,iq,QT,
     s      XSF,MSF,DSF,XSD,MSD,DSD,QINT,QGEQ,VSW,MSW,DSW,RHOW)

C------------------------------------------------------------------------
C     
C     -- DESCRIPTION 
C     
C     This subroutine computes the gas/aerosol equilibrium.
C     
C------------------------------------------------------------------------
C     
C     -- INPUT VARIABLES
C     
C     IS1 : first bin to compute.
C     IS2 : last bin to compute.
C     NEQ : number of equations.
C     Q : aerosol concentration ([g.m-3]).
C     
C     
C     -- INPUT/OUTPUT VARIABLES
C     
C     
C     -- OUTPUT VARIABLES
C     
C------------------------------------------------------------------------
C     
C     -- REMARKS
C     
C------------------------------------------------------------------------
C     
C     -- MODIFICATIONS
C     
C     2005/3/23: cleaning (Bruno Sportisse, CEREA).
C     
C------------------------------------------------------------------------
C     
C     -- AUTHOR(S)
C     
C     2004: Edouard Debry, CEREA.
C     
C     
C------------------------------------------------------------------------

      IMPLICIT NONE

      INCLUDE 'param.inc'
      INCLUDE 'pointer.inc'
      INCLUDE 'dynaero.inc'
      INCLUDE 'varp.inc'
      
      INTEGER is1,is2,neq,nbin_aer
      DOUBLE PRECISION q(neq)

      INTEGER js,jj,jesp,iq(NEXT,nbin_aer)
      DOUBLE PRECISION qext(NEXT)

      DOUBLE PRECISION QGEQTMPA(NEXT),QINTMPA(NINTIS)
            
      DOUBLE PRECISION XSF(nbin_aer),MSF(nbin_aer),DSF(nbin_aer)
      DOUBLE PRECISION XSD(nbin_aer),MSD(nbin_aer),DSD(nbin_aer)

      DOUBLE PRECISION QT(nbin_aer)
      DOUBLE PRECISION VSW(nbin_aer),MSW(nbin_aer)
      DOUBLE PRECISION QINT(NINTIS,nbin_aer),DSW(nbin_aer)
      DOUBLE PRECISION QGEQ(NEXT,nbin_aer),RHOW(nbin_aer)
      
      DO js=is1,is2
                                ! zero init
         DO jesp=1,NEXT
            qext(jesp)=0.D0
         END DO

                                ! compute total dry mass
         QT(js)=0.D0
         DO jesp=E1,E2
            jj=IQ(jesp,js)
            qext(jesp)=q(jj)
            QT(js)=QT(js)+qext(jesp) ! g.m-3
         END DO

         IF ( q(js).GT.TINYN.AND.
     &        QT(js).GT.TINYM ) THEN

            CALL STEP( q(js),   ! num aero conc (#part.m-3)
     &           qext,          ! extern aero conc (g.m-3)
     &           QT(js),        ! tot dry conc (g.m-3)
     &           QINTMPA,       ! int inorg conc (g.m-3)
     &           QGEQTMPA,      ! equi gas conc (g.m-3)
     &           VSW(js),       ! vol aero conc (m3.m-3)
     &           DSD(js),       ! dry aero diam (m)
     &           DSW(js),       ! wet aero diam (m)
     &           RHOW(js))      ! aerosol density (g/m-3)

            DO jj=1,NEXT
               QGEQ(jj,js) = QGEQTMPA(jj)
            ENDDO

            DO jj=1,NINTIS
               QINT(jj,js) = QINTMPA(jj) 
            ENDDO

            jj=IQ(EH2O,js)
            q(jj)=qext(EH2O)
                                ! qn cannot be less than tinyn
            MSD(js)=QT(js)/q(js) ! single dry aero mass (g)
            XSD(js)=DLOG(MSD(js)) ! dry log mass (adim)
            MSW(js)=MSD(js)+q(jj)/q(js) ! single wet mass (g)

         ELSE
                                ! if too few aerosols or too few mass
                                ! we set variables of given bins as
                                ! its initial fixed ones, 
                                ! thus avoiding zero values
            MSD(js)=MSF(js)
            DSD(js)=DSF(js)
            XSD(js)=XSF(js)
            MSW(js)=MSF(js)
            DSW(js)=DSF(js)
            RHOW(js)=RHOA
         ENDIF
      END DO

      END
