C------------------------------------------------------------------------
C     
C     Copyright (C) 2003-2006 CEREA (ENPC)
C     
C     This file is part of Polair.
C     Polair is a 3D eulerian chemistry-transport-model.
C     
C     Polair is free software; you can redistribute it and/or modify
C     it under the terms of the GNU General Public License as published by
C     the Free Software Foundation; either version 2 of the License, or
C     (at your option) any later version.
C     
C     Polair is distributed in the hope that it will be useful,
C     but WITHOUT ANY WARRANTY; without even the implied warranty of
C     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C     GNU General Public License (file "LICENSE") for more details.
C     
C------------------------------------------------------------------------

      SUBROUTINE FASTDIAM(nbin_aer,is1,is2,neq,q,iq,QT,
     s      XSF,MSF,DSF,XSD,MSD,DSD,VSW,MSW,DSW,RHOW)

C------------------------------------------------------------------------
C     
C     -- DESCRIPTION 
C     
C     This subroutine fastly computes dry and wet aerosol diameters  
C     
C------------------------------------------------------------------------
C     
C     -- INPUT VARIABLES
C     
C     IS1 : first bin to compute.
C     IS2 : last bin to compute.
C     NEQ : number of equations.
C     Q : aerosol concentration ([g.m-3]).
C     
C     
C     -- INPUT/OUTPUT VARIABLES
C     
C     
C     -- OUTPUT VARIABLES
C     
C------------------------------------------------------------------------
C     
C     -- REMARKS
C     
C------------------------------------------------------------------------
C     
C     -- MODIFICATIONS
C     
C------------------------------------------------------------------------
C     
C     -- AUTHOR(S)
C     
C     2005: Edouard Debry, CEREA.
C     
C     
C------------------------------------------------------------------------

      IMPLICIT NONE

      INCLUDE 'param.inc'
      INCLUDE 'CONST_A.INC'
      INCLUDE 'pointer.inc'
      INCLUDE 'dynaero.inc'
      INCLUDE 'varp.inc'
      INCLUDE 'meteo.inc'
      INCLUDE 'liqmd.inc'
      
      INTEGER is1,is2,neq,nbin_aer
      DOUBLE PRECISION q(neq)

      INTEGER js,jj,jesp,iq(NEXT,nbin_aer)
      DOUBLE PRECISION qext(NEXT),vad,qn

      DOUBLE PRECISION XSF(nbin_aer),MSF(nbin_aer),DSF(nbin_aer)
      DOUBLE PRECISION XSD(nbin_aer),MSD(nbin_aer),DSD(nbin_aer)

      DOUBLE PRECISION QT(nbin_aer)
      DOUBLE PRECISION VSW(nbin_aer),MSW(nbin_aer)
      DOUBLE PRECISION DSW(nbin_aer),RHOW(nbin_aer)

      DO js=is1,is2
                                ! zero init
         DO jesp=1,NEXT
            qext(jesp)=0.D0
         END DO

                                ! compute total dry mass
         QT(js)=0.D0
         DO jesp=E1,E2
            jj=IQ(jesp,js)
            qext(jesp)=q(jj)
            QT(js)=QT(js)+qext(jesp) ! g.m-3
         END DO
     
         IF ( q(js).GT.TINYN.AND.
     &        QT(js).GT.TINYM ) THEN

            qn=q(js)            ! num aero conc (#part.m-3)
            RHOW(js)=RHOA       ! fixed aerosol density (g.m-3)

            vad=QT(js)/RHOW(js)     ! dry aerosol volume (m3.m-3)
            DSD(js)=(vad/qn/cst_pi6)**cst_FRAC3 ! m, cannot be less than tinyn
            
            CALL gerber_wet_diameter(RH, ! relative humidity (0<adim<1)
     &           TEMP,          ! temperature (Kelvin)
     &           DSD(js),      ! dry aero diam (m)
     &           DSW(js))      ! wet aero diam (m)

            VSW(js)=qn*cst_pi6*DSW(js)*DSW(js)*DSW(js) ! wet aerosol volume (m3.m-3)
            qext(EH2O)=DMAX1(VSW(js)-vad,0.D0)*LMD(EH2O) ! water conc (g.m-3)

            jj=IQ(EH2O,js)
            q(jj)=qext(EH2O)

            MSD(js)=QT(js)/q(js) ! single dry aero mass (g)
            XSD(js)=DLOG(MSD(js)) ! dry log mass (adim)
            MSW(js)=MSD(js)+q(jj)/q(js) ! single wet mass (g)

         ELSE
                                ! if too few aerosols or too few mass
                                ! we set variables of given bins as
                                ! its initial fixed ones, 
                                ! thus avoiding zero values
            RHOW(js)=RHOA       ! fixed aerosol density (g.m-3)
            MSD(js)=MSF(js)
            DSD(js)=DSF(js)
            XSD(js)=XSF(js)
            MSW(js)=MSF(js)
            DSW(js)=DSF(js)

         ENDIF
      END DO

      END

