C------------------------------------------------------------------------
C     
C     Copyright (C) 2003-2006 CEREA (ENPC)
C     
C     This file is part of Polair.
C     Polair is a 3D eulerian chemistry-transport-model.
C     
C     Polair is free software; you can redistribute it and/or modify
C     it under the terms of the GNU General Public License as published by
C     the Free Software Foundation; either version 2 of the License, or
C     (at your option) any later version.
C     
C     Polair is distributed in the hope that it will be useful,
C     but WITHOUT ANY WARRANTY; without even the implied warranty of
C     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C     GNU General Public License (file "LICENSE") for more details.
C     
C------------------------------------------------------------------------

      SUBROUTINE FCOND(neq,nbin_aer,q,iq,dqdt,QT,
     s      XSF,MSF,DSF,XSD,MSD,DSD,AA)

C------------------------------------------------------------------------
C     
C     -- DESCRIPTION 
C     
C     This subroutine computes the source terms for the system of
C     Ordinary Differential Equations defined by condensation/evaporation
C     along a moving grid.    
C     
C------------------------------------------------------------------------
C     
C     -- INPUT VARIABLES
C     
C     NEQ : number of equations.
C     Q   : aerosol concentration ([g.m-3]).
C     
C     -- INPUT/OUTPUT VARIABLES
C     
C     
C     -- OUTPUT VARIABLES
C     
C     DQDT : time derivative ([g.m-3.s-1]).
C     
C------------------------------------------------------------------------
C     
C     -- REMARKS
C     
C------------------------------------------------------------------------
C     
C     -- MODIFICATIONS
C     
C     2005/3/23: cleaning (Bruno Sportisse, CEREA).
C     
C------------------------------------------------------------------------
C     
C     -- AUTHOR(S)
C     
C     2004: Edouard Debry, CEREA.
C     
C------------------------------------------------------------------------

      IMPLICIT NONE

      INCLUDE 'param.inc'
      INCLUDE 'dynaero.inc'
      INCLUDE 'pointer.inc'
      INCLUDE 'varp.inc'
      
      INTEGER neq,nbin_aer
      DOUBLE PRECISION q(neq),dqdt(neq)

      INTEGER js,jj,jesp,GLOC,iq(NEXT,nbin_aer)

      DOUBLE PRECISION XSF(nbin_aer),MSF(nbin_aer),DSF(nbin_aer)
      DOUBLE PRECISION XSD(nbin_aer),MSD(nbin_aer),DSD(nbin_aer)      

      DOUBLE PRECISION QT(nbin_aer)
      DOUBLE PRECISION VSW(nbin_aer),MSW(nbin_aer)
      DOUBLE PRECISION QINT(NINTIS,nbin_aer),DSW(nbin_aer)
      DOUBLE PRECISION QGEQ(NEXT,nbin_aer),RHOW(nbin_aer)

      DOUBLE PRECISION KERCD(NEXT,nbin_aer),CKV(NEXT,nbin_aer)
      DOUBLE PRECISION AA(NEXT,nbin_aer)

C     Compute gas mass conservation.

      CALL MASSCNSRV(neq,nbin_aer,q,iq)

C     Compute c/e kernels in dynamic section.
      INORG2=INORG
      IORG2=IORG
      CALL EQPART(nbin_aer,ICUT2+1,nbin_aer,neq,q,iq,QT,
     s      XSF,MSF,DSF,XSD,MSD,DSD,
     s      QINT,QGEQ,VSW,MSW,DSW,RHOW)
      INORG2=INORG
      IORG2=IORG
      IKV2=IKELV
      CALL KERCD2(nbin_aer,ICUT2+1,nbin_aer,neq,q,iq,QT,
     s      QINT,QGEQ,VSW,DSW,AA,CKV,KERCD)

C     Update time derivatives.

      IF(ISULFCOND.EQ.1) THEN
         GLOC = G12           ! Do not take sulfate into account
      ELSE
         GLOC = G1
      ENDIF


      DO jesp=GLOC,G2
         DO js=ICUT2+1,nbin_aer
            jj=IQ(jesp,js)
            dqdt(jj)=dqdt(jj)+KERCD(jesp,js)*q(js)
         END DO
      END DO

      END
