C------------------------------------------------------------------------
C     
C     Copyright (C) 2003-2006 CEREA (ENPC)
C     
C     This file is part of Polair.
C     Polair is a 3D eulerian chemistry-transport-model.
C     
C     Polair is free software; you can redistribute it and/or modify
C     it under the terms of the GNU General Public License as published by
C     the Free Software Foundation; either version 2 of the License, or
C     (at your option) any later version.
C     
C     Polair is distributed in the hope that it will be useful,
C     but WITHOUT ANY WARRANTY; without even the implied warranty of
C     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C     GNU General Public License (file "LICENSE") for more details.
C     
C------------------------------------------------------------------------

      SUBROUTINE FGDE(neq,nbin_aer,ZSC,q,iq,dqdt,couples_coag,
     s      first_index_coag,second_index_coag,
     s      coefficient_coag,QT,XSF,MSF,DSF,XSD,MSD,DSD,AA)

C------------------------------------------------------------------------
C     
C     -- DESCRIPTION 
C     
C     This subroutine computes the source terms for the system of
C     Ordinary Differential Equations defined by the GDE.    
C     
C------------------------------------------------------------------------
C     
C     -- INPUT VARIABLES
C     
C     NEQ : number of equations.
C     Q   : aerosol concentration ([g.m-3]).
C     ZSC: volumic emissions for aerosol ([\mu g.m-3.s-1]).
C     
C     -- INPUT/OUTPUT VARIABLES
C     
C     
C     -- OUTPUT VARIABLES
C     
C     DQDT : time derivative ([g.m-3.s-1]).
C     
C------------------------------------------------------------------------
C     
C     -- REMARKS
C     
C------------------------------------------------------------------------
C     
C     -- MODIFICATIONS
C     
C     2005/3/23: cleaning (Bruno Sportisse, CEREA).
C     
C------------------------------------------------------------------------
C     
C     -- AUTHOR(S)
C     
C     2004: Edouard Debry, CEREA.
C     
C------------------------------------------------------------------------

      IMPLICIT NONE

      INCLUDE 'param.inc'
      INCLUDE 'varp.inc'
      
      INTEGER neq,nbin_aer
      DOUBLE PRECISION ZSC(neq),q(neq),dqdt(neq)
      INTEGER iq(NEXT,nbin_aer)
      INTEGER couples_coag(nbin_aer)
      INTEGER first_index_coag(nbin_aer, 4 * nbin_aer)
      INTEGER second_index_coag(nbin_aer, 4 * nbin_aer)
      DOUBLE PRECISION coefficient_coag(nbin_aer, nbin_aer, nbin_aer)

      DOUBLE PRECISION XSF(nbin_aer),MSF(nbin_aer),DSF(nbin_aer)
      DOUBLE PRECISION XSD(nbin_aer),MSD(nbin_aer),DSD(nbin_aer)

      DOUBLE PRECISION QT(nbin_aer)

      DOUBLE PRECISION AA(NEXT,nbin_aer)

      INTEGER jj
      
C     Initialization to zero.

      DO jj=1,neq
         dqdt(jj)=0.D0
      END DO

C     Compute the physical processes

      IF (ICG2.EQ.1) CALL FCOAG(neq,nbin_aer,q,iq,dqdt,couples_coag,
     s      first_index_coag,second_index_coag,coefficient_coag,QT,
     s      XSF,MSF,DSF,XSD,MSD,DSD)
      
      IF (ICE2.EQ.1) CALL FCOND(neq,nbin_aer,q,iq,dqdt,QT,
     s      XSF,MSF,DSF,XSD,MSD,DSD,AA)

      IF (INU2.EQ.1) CALL FNUCL(neq,nbin_aer,q,iq,dqdt)

      IF (ISC2.EQ.1) then
         DO jj=1,neq
            dqdt(jj)=dqdt(jj) + ZSC(jj)
         END DO
      Endif
      

      END

