C------------------------------------------------------------------------
C     
C     Copyright (C) 2003-2006 CEREA (ENPC)
C     
C     This file is part of Polair.
C     Polair is a 3D eulerian chemistry-transport-model.
C     
C     Polair is free software; you can redistribute it and/or modify
C     it under the terms of the GNU General Public License as published by
C     the Free Software Foundation; either version 2 of the License, or
C     (at your option) any later version.
C     
C     Polair is distributed in the hope that it will be useful,
C     but WITHOUT ANY WARRANTY; without even the implied warranty of
C     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C     GNU General Public License (file "LICENSE") for more details.
C     
C------------------------------------------------------------------------

      SUBROUTINE INITSTEP(neq,nbin_aer,ZSC,q,iq,couples_coag,
     s      first_index_coag,second_index_coag,
     s      coefficient_coag,QT,XSF,MSF,DSF,XSD,MSD,DSD)

C------------------------------------------------------------------------
C     
C     -- DESCRIPTION 
C     
C     This subroutine performs the initialization of the timestep
C     for the integration of the GDE. The criterion is related to
C     the timescales of the aerosol processes.
C------------------------------------------------------------------------
C     
C     -- INPUT VARIABLES
C     
C     NEQ : number of equations.
C     Q   : gas/aerosol concentration ([g.m-3]).
C     ZSC: volumic emissions for aerosol ([\mu g.m-3.s-1]).
C     
C     -- INPUT/OUTPUT VARIABLES
C     
C     
C     -- OUTPUT VARIABLES
C     
C------------------------------------------------------------------------
C     
C     -- REMARKS
C     
C------------------------------------------------------------------------
C     
C     -- MODIFICATIONS
C     
C     2005/3/23: cleaning (Bruno Sportisse, CEREA).
C     
C------------------------------------------------------------------------
C     
C     -- AUTHOR(S)
C     
C     2004: Edouard Debry, CEREA.
C     
C------------------------------------------------------------------------

      IMPLICIT NONE
      
      INCLUDE 'param.inc'
      INCLUDE 'dynaero.inc'
      INCLUDE 'pointer.inc'
      INCLUDE 'time.inc'
      INCLUDE 'varp.inc'  

      INTEGER neq,nbin_aer
      DOUBLE PRECISION q(neq),ZSC(neq)
      INTEGER iq(NEXT,nbin_aer)
      INTEGER couples_coag(nbin_aer)
      INTEGER first_index_coag(nbin_aer, 4 * nbin_aer)
      INTEGER second_index_coag(nbin_aer, 4 * nbin_aer)
      DOUBLE PRECISION coefficient_coag(nbin_aer, nbin_aer, nbin_aer)

      DOUBLE PRECISION XSF(nbin_aer),MSF(nbin_aer),DSF(nbin_aer)
      DOUBLE PRECISION XSD(nbin_aer),MSD(nbin_aer),DSD(nbin_aer)

      INTEGER jj
      DOUBLE PRECISION tscale,tmp,dqdt(neq)

      DOUBLE PRECISION QT(nbin_aer)

      DOUBLE PRECISION AA(NEXT,nbin_aer)

C     Time step set to output time.

      TCG=TOUT
      TCE=TOUT
      TSC=TOUT

C     Coagulation time step.

      IF (ICOAG.EQ.1) THEN
         ICG2=1
         ICE2=0
         INU2=0
         ISC2=0
         CALL FGDE(neq,nbin_aer,ZSC,q,iq,dqdt,couples_coag,
     s      first_index_coag,second_index_coag,
     s      coefficient_coag,QT,XSF,MSF,DSF,XSD,MSD,DSD,AA)
         
         DO jj=1,neq
            tmp=q(jj)*dqdt(jj)
            IF (tmp.NE.0.D0) THEN
               tscale=q(jj)/DABS(dqdt(jj))
               TCG=DMIN1(TCG,tscale)                 
            ENDIF
         END DO
      ENDIF
      
C     Cond/evap time step.

      IF (ICOND.EQ.1) THEN
         ICG2=0
         ICE2=1
         INU2=INUCL             
         ICUT2=ICUT             ! current cutting size
         ISC2=0
         CALL FGDE(neq,nbin_aer,ZSC,q,iq,dqdt,couples_coag,
     s      first_index_coag,second_index_coag,
     s      coefficient_coag,QT,XSF,MSF,DSF,XSD,MSD,DSD,AA)
         
         DO jj=1,neq
            tmp=q(jj)*dqdt(jj)
            IF (tmp.NE.0.D0) THEN
               
               tscale=q(jj)/DABS(dqdt(jj))
               TCE=DMIN1(TCE,tscale)
            ENDIF
         END DO
      ENDIF
      
      IF(ISOURC.EQ.1) THEN
         ICG2=0
         ICE2=0
         INU2=0            
         ICUT2=0   
         ISC2=1                 ! Volumic emissions
         CALL FGDE(neq,nbin_aer,ZSC,q,iq,dqdt,couples_coag,
     s      first_index_coag,second_index_coag,
     s      coefficient_coag,QT,XSF,MSF,DSF,XSD,MSD,DSD,AA)
         
         DO jj=1,neq
            tmp=q(jj)*dqdt(jj)
            IF (tmp.NE.0.D0) THEN
               
               tscale=q(jj)/DABS(dqdt(jj))
               TSC=DMIN1(TSC,tscale)
            ENDIF
         END DO
      ENDIF

C     Minimal time step.
                                ! it is assumed that coagulation is 
                                ! always the slowest process
      DT=TCG
      DT=DMIN1(DT,TOUT-TIN)
      

                                ! if only one process then set
                                ! splitting step to whole timestep     

      DT = DMAX1(DT,DTAEROMIN)
      TCE = DMIN1(DT,DMAX1(TCE,DTAEROMIN))
      TCG = DMIN1(DT,DMAX1(TCG,DTAEROMIN))
      TSC = DMIN1(DT,DMAX1(TSC,DTAEROMIN))
      IF(ICOAG.EQ.0.OR.ICOND.EQ.0)  DT=TOUT-TIN     
      IF(ICUT.EQ.nbin_aer) DT=TOUT-TIN
      
      END

