C------------------------------------------------------------------------
C     
C     Copyright (C) 2003-2006 CEREA (ENPC)
C     
C     This file is part of Polair.
C     Polair is a 3D eulerian chemistry-transport-model.
C     
C     Polair is free software; you can redistribute it and/or modify
C     it under the terms of the GNU General Public License as published by
C     the Free Software Foundation; either version 2 of the License, or
C     (at your option) any later version.
C     
C     Polair is distributed in the hope that it will be useful,
C     but WITHOUT ANY WARRANTY; without even the implied warranty of
C     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C     GNU General Public License (file "LICENSE") for more details.
C     
C------------------------------------------------------------------------

      SUBROUTINE MASSCNSRV(neq,nbin_aer,q,iq)

C------------------------------------------------------------------------
C     
C     -- DESCRIPTION 
C     
C     This subroutine computes the gas-phase concentration by mass
C     conservation.     
C     
C------------------------------------------------------------------------
C     
C     -- INPUT VARIABLES
C     
C     NEQ : number of equations.
C     
C     
C     -- INPUT/OUTPUT VARIABLES
C     
C     Q : gas/aerosol concentration ([g.m-3]).
C     
C     
C     -- OUTPUT VARIABLES
C     
C------------------------------------------------------------------------
C     
C     -- REMARKS
C     
C------------------------------------------------------------------------
C     
C     -- MODIFICATIONS
C     
C     2005/3/23: cleaning (Bruno Sportisse, CEREA).
C     
C------------------------------------------------------------------------
C     
C     -- AUTHOR(S)
C     
C     2004: Edouard Debry, CEREA.
C     
C------------------------------------------------------------------------

      IMPLICIT NONE

      INCLUDE 'param.inc'
      INCLUDE 'dynaero.inc'
      INCLUDE 'varq.inc' 
      INCLUDE 'varp.inc'
      
      INTEGER neq,nbin_aer
      DOUBLE PRECISION q(neq)
      INTEGER iq(NEXT,nbin_aer)

      INTEGER js,ji,jj,jesp
      DOUBLE PRECISION qaero

C     ******Compute gas conc by mass conservation.

      DO jesp=G1,G2
         ji=IG(jesp)

         qaero=0.D0
         DO js=1,nbin_aer
            jj=IQ(jesp,js)
            qaero=qaero+q(jj)
         END DO
         
         q(ji)=QTOT(jesp)-qaero 

         IF(q(ji) .lt. 0.d0) then
            DO js = 1,nbin_aer
               q(IQ(jesp,js)) = q(IQ(jesp,js))-(dabs(q(ji)))*
     &              q(IQ(jesp,js))/qaero
            ENDDO

            qaero=0.D0
            DO js=1,nbin_aer
               jj=IQ(jesp,js)
               qaero=qaero+q(jj)
            END DO
            
            q(ji)=QTOT(jesp)-qaero 
         ENDIF

      END DO

      END

