C------------------------------------------------------------------------
C     
C     Copyright (C) 2003-2006 CEREA (ENPC)
C     
C     This file is part of Polair.
C     Polair is a 3D eulerian chemistry-transport-model.
C     
C     Polair is free software; you can redistribute it and/or modify
C     it under the terms of the GNU General Public License as published by
C     the Free Software Foundation; either version 2 of the License, or
C     (at your option) any later version.
C     
C     Polair is distributed in the hope that it will be useful,
C     but WITHOUT ANY WARRANTY; without even the implied warranty of
C     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C     GNU General Public License (file "LICENSE") for more details.
C     
C------------------------------------------------------------------------

      SUBROUTINE PROCESSAERO(neq,nbin_aer,ZSC,q,iq,couples_coag,
     s      first_index_coag,second_index_coag,
     s      coefficient_coag,QT,XSF,MSF,DSF,XSD,MSD,DSD)

C------------------------------------------------------------------------
C     
C     -- DESCRIPTION 
C     
C     This subroutine integrates the different processes for the GDE.     
C     
C------------------------------------------------------------------------
C     
C     -- INPUT VARIABLES
C     
C     NEQ : number of equations.
C     Q   : gas/aerosol concentrations ([\mu.g.m^-3]).
C     ZSC: volumic emissions for aerosol ([\mu g.m^-3.s^-1])     
C     
C     -- INPUT/OUTPUT VARIABLES
C     
C     
C     -- OUTPUT VARIABLES
C     
C------------------------------------------------------------------------
C     
C     -- REMARKS
C     
C------------------------------------------------------------------------
C     
C     -- MODIFICATIONS
C     
C     2005/3/23: cleaning (Bruno Sportisse, CEREA).
C     2005/9/30: Add possibility to compute sulfate cond. explicitely
C     + Remove full eq. solver from FGDE (Karine Sartelet, CEREA).
C     
C------------------------------------------------------------------------
C     
C     -- AUTHOR(S)
C     
C     2004: Edouard Debry, CEREA.
C     
C------------------------------------------------------------------------

      IMPLICIT NONE

      INCLUDE 'param.inc'
      INCLUDE 'dynaero.inc'
      INCLUDE 'time.inc'
      INCLUDE 'varp.inc'
      
      INTEGER neq,nbin_aer
      DOUBLE PRECISION q(neq),q1(neq)
      DOUBLE PRECISION ZSC(neq)
      INTEGER iq(NEXT,nbin_aer)

      INTEGER couples_coag(nbin_aer)
      INTEGER first_index_coag(nbin_aer, 4 * nbin_aer)
      INTEGER second_index_coag(nbin_aer, 4 * nbin_aer)
      DOUBLE PRECISION coefficient_coag(nbin_aer, nbin_aer, nbin_aer)
 
      DOUBLE PRECISION XSF(nbin_aer),MSF(nbin_aer),DSF(nbin_aer)
      DOUBLE PRECISION XSD(nbin_aer),MSD(nbin_aer),DSD(nbin_aer)

      DOUBLE PRECISION QT(nbin_aer)

C     ******time loop
      DO WHILE ( TIN2 .LT. TOUT2 )

         IF (ICE2.EQ.1) THEN
C     Compute gas mass conservation.
            CALL MASSCNSRV(neq,nbin_aer,q,iq)

C     solve sulfate dynamically
            IF (ISULFCOND.EQ.1) CALL SULFDYN(neq,nbin_aer,q,iq,QT,
     &           XSF,MSF,DSF,XSD,MSD,DSD)

C     solve equilibrium with
C     so-called bulk approach
            IF (ICUT2.GT.0) CALL BULKEQUI(neq,nbin_aer,q,iq,QT,
     s           XSF,MSF,DSF,XSD,MSD,DSD)
         ENDIF
            
         
         IF(ICUT2.EQ.nbin_aer.AND.
     s        INU2.EQ.0.AND.
     s        ICG2.EQ.0.AND.
     s        ISC2.EQ.0) THEN
            DT2 = TOUT2 - TIN2
            TIN2 = TOUT2
         ElSEIF (KDSLV2.EQ.1) THEN
                                ! etr dynamic solver
 
            CALL ETRCONC(neq,nbin_aer,ZSC,q1,q,iq,couples_coag,
     s      first_index_coag,second_index_coag,
     s      coefficient_coag,QT,XSF,MSF,DSF,XSD,MSD,DSD)
            
         ELSEIF (KDSLV2.EQ.2) THEN
                                ! ros2 dynamic solver

            CALL ROS2CONC(neq,nbin_aer,ZSC,q1,q,iq,couples_coag,
     s      first_index_coag,second_index_coag,
     s      coefficient_coag,QT,XSF,MSF,DSF,XSD,MSD,DSD)

         ELSEIF (KDSLV2.EQ.3) THEN
                                ! ebi dynamic solver

            CALL EBICONC(neq,nbin_aer,q1,q,iq,QT,
     s      XSF,MSF,DSF,XSD,MSD,DSD)
         ENDIF

                                ! adaptative time step
C     check the need to call ADAPTSTEP (not for the last timestep)         
         IF (TIN2.LT.TOUT2 ) THEN
            CALL ADAPTSTEP(neq,nbin_aer,q1,q,iq)
         ENDIF

      END DO

      END

