C------------------------------------------------------------------------
C     
C     Copyright (C) 2003-2006 CEREA (ENPC)
C     
C     This file is part of Polair.
C     Polair is a 3D eulerian chemistry-transport-model.
C     
C     Polair is free software; you can redistribute it and/or modify
C     it under the terms of the GNU General Public License as published by
C     the Free Software Foundation; either version 2 of the License, or
C     (at your option) any later version.
C     
C     Polair is distributed in the hope that it will be useful,
C     but WITHOUT ANY WARRANTY; without even the implied warranty of
C     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C     GNU General Public License (file "LICENSE") for more details.
C     
C------------------------------------------------------------------------

      SUBROUTINE VOLAERO(qext,qinti,rhoaer)

C------------------------------------------------------------------------
C     
C     -- DESCRIPTION 
C     
C     This subroutine computes the dry and wet aerosol volumes and the
C     aerosol density according to the internal composition.    
C     
C------------------------------------------------------------------------
C     
C     -- INPUT VARIABLES
C     
C     
C     QEXT :  external aerosol concentration   ([\mu.g.m^-3]).
C     QINTI:  internal inorganic concentration ([\mu.g.m^-3]).
C     
C     -- INPUT/OUTPUT VARIABLES
C     
C     
C     -- OUTPUT VARIABLES
C     
C     RHOAER : aerosol density ([\mu.g.\mu.m^-3]).
C
C------------------------------------------------------------------------
C     
C     -- REMARKS
C     
C------------------------------------------------------------------------
C     
C     -- MODIFICATIONS
C     
C     2005/3/23: cleaning (Bruno Sportisse, CEREA).
C     
C------------------------------------------------------------------------
C     
C     -- AUTHOR(S)
C     
C     2004: Edouard Debry, CEREA.
C     
C------------------------------------------------------------------------

      IMPLICIT NONE

      INCLUDE 'param.inc'
      INCLUDE 'pointer.inc'
      INCLUDE 'dynaero.inc'
      INCLUDE 'liqmd.inc'
      INCLUDE 'solmd.inc'
      INCLUDE 'CONST.INC'
      INCLUDE 'CONST_A.INC'

      DOUBLE PRECISION qext(NEXT),qinti(NINTIS),rhoaer

      INTEGER jesp
      DOUBLE PRECISION vid,viw,vis,vil,vod,vad,vaw,sumint
      
C     ******inorganic volume
      vid=0.D0
      vis=0.D0
      vil=0.D0
      viw=0.D0
      vod=0.D0
      sumint=0.d0

C     Mineral Dust
      vis = vis + qext(EMD)/LMD(EMD)
      sumint = sumint + qext(EMD)

C     Black Carbon
      vis = vis + qext(EBC)/LMD(EBC)
      sumint = sumint + qext(EBC)

C     Inorganic
                                ! compute solid aerosol volume
      DO jesp=SNaNO3,SLC
         vis=vis+qinti(jesp)
     &        /SMD(jesp)
      END DO

                                ! compute liquid aerosol volume

                                ! sodium volume
      vil=vil + qinti(INa)/LMD(ENa)

                                ! ammonium volume
      vil=vil+( qinti(INH3)
     &     +qinti(INH4)
     &     *0.944444444444D0    ! mwnh3/mwinh4
     &     )/LMD(ENH3)          ! g.m-3

                                ! nitric acid volume
      vil=vil+( qinti(IHNO3)
     &     +qinti(INO3)
     &     *1.01612903226D0     ! mwhno3/mwino3
     &     )/LMD(ENO3)          ! g.m-3

                                ! chlorhydric acid volume
      vil=vil+( qinti(IHCl)
     &     +qinti(ICl)
     &     *1.02816901408D0     ! mwhcl/mwicl
     &     )/LMD(ECl)           ! g.m-3

                                ! sulfuric acid volume
      vil=vil+( qinti(IHSO4)
     &     *1.01030927835D0     ! mwh2so4/mwihso4
     &     +qinti(ISO4)
     &     *1.02083333333D0     ! mwh2so4/mwiso4
     &     )/LMD(ESO4)          ! g.m-3
      
      vid=vil+vis               ! dry inorg vol m3.m-3

                                ! water volume
      viw=qext(EH2O)/LMD(EH2O)

                                ! total inorg internal mass

      DO jesp=1,NINTIS
         sumint=sumint+qinti(jesp)
      ENDDO

                                ! correction for water         
      sumint=sumint-qinti(IH2O)-qinti(IOH)+qext(EH2O)
      
C     dry organic volume and total internal mass
      IF (IORG.EQ.1) THEN
         DO jesp=EARO1,EPOA
            vod=vod+qext(jesp)/LMD(jesp)
            sumint=sumint+qext(jesp)
         END DO
      ENDIF

C     ******tot dry and wet vol, m3.m-3
      vad=vod+vid
      vaw=vad+viw
      
C     Notice that the density is based on the internal composition
C     and as such bigger than the minimal density and less that the maximal
C     one 

      IF ((vaw.GT.0.d0).AND.(sumint.GT.0.D0)) THEN
         rhoaer=sumint/vaw
      ELSE
         rhoaer=RHOA
      ENDIF

      END

