// Copyright (C) 2005 CEREA
//     Author: Vivien Mallet
//
// CEREA (http://www.enpc.fr/cerea) is a joint laboratory of
// ENPC (http://www.enpc.fr) and EDF R&D (http://www.edf.fr).
//
// This file is part of the chemistry-transport model Polair3D.
//
// This code is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This file is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License (file "license") for more details.


#ifndef POLYPHEMUS_FILE_MODULES_TRANSPORT_ADVECTIONDST3_HXX


#include <vector>
#include "AtmoData.hxx"


namespace Polyphemus
{


  using namespace std;
  using namespace AtmoData;


  //////////////////////
  // FORTRAN FUNCTION //
  //////////////////////


#define _advection advection_

  extern "C"
  {
    void _advection(int*, int*, int*, double*, double*,
                    double*, double*, double*, double*, int*,
		    double*, double*, double*, double*, double*);
  }


  ///////////////////
  // ADVECTIONDST3 //
  ///////////////////


  //! This class is a numerical solver for advection.
  /*! It uses a Direct-Space-Time (DST) algorithm of third order with a
    Koren-like flux limiter.
   */
  template<class T>
  class AdvectionDST3
  {

  public:

    template<class ClassModel>
    void Init(ClassModel& Model);
    
    template<class ClassModel>
    void Forward(ClassModel& Model);
    template<class ClassModel>
    void Forward_aer(ClassModel& Model);

    void Forward(Data<T, 3>& ZonalWind, Data<T, 3>& MeridionalWind,
		 Data<T, 3>& VerticalWind, int with_bc,
		 Array<T, 1>& CellWidth_x, Array<T, 1>& CellWidth_y,
		 Array<T, 1>& CellWidth_z, int Nx,int Ny, int Nz,
		 Data<T, 3>& BoundaryCondition_x,
		 Data<T, 3>& BoundaryCondition_y,
		 Data<T, 2>& BoundaryCondition_z, T Delta_t,
		 Data<T, 3>& Concentration);

  };


} // namespace Polyphemus.


#define POLYPHEMUS_FILE_MODULES_TRANSPORT_ADVECTIONDST3_HXX
#endif
