// Copyright (C) 2003-2005 CEREA
//     Author: Vivien Mallet
//
// CEREA (http://www.enpc.fr/cerea) is a joint laboratory of
// ENPC (http://www.enpc.fr) and EDF R&D (http://www.edf.fr).
//
// This file is part of a simulation system for air quality.
//
// This code is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This file is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License (file "license") for more details.


// This program computes the roughness on the output grid on the basis of
// roughness data given by land use coverage categories.


//////////////
// INCLUDES //

#include <vector>
using namespace std;

#define SELDONDATA_DEBUG_LEVEL_4

#include "AtmoData.hxx"
using namespace AtmoData ;

// INCLUDES //
//////////////


int main(int argc, char **argv)
{
  
  TRY;

  cout << endl;

  string configuration_file(""), sec_config_file("");

  if (argc != 2 && argc != 3)
    {
      string mesg  = "Usage:\n";
      mesg += string("  ") + argv[0] + " [main configuration file] [secondary config file] [date]\n";
      mesg += string("  ") + argv[0] + " [main configuration file] \n";
      mesg += "Arguments:\n";
      mesg += "  [main configuration file] (optional): main configuration file.\n";
      mesg += "  [secondary configuration file] (optional): secondary configuration file.\n";
      cout << mesg << endl;
      return 1;
    }

  if (argc == 3)
    sec_config_file = argv[2];
  if (argc == 2 || argc == 3)
    configuration_file = argv[1];
  
  // Configuration files.
  if (!exists(configuration_file))
    throw string("Unable to find configuration file \"")
      + configuration_file + "\".";
  

  ////////////////////////
  // FIRST DECLARATIONS //
  ////////////////////////

  typedef float real;

  int i, j, k;
  
  // Input domain.
  int Ny, Nx, Nc;

  // Files.
  string dir_out, file_out, LUC_file, roughness_data_file;


  /////////////////////////
  // CONFIGURATION FILES //
  /////////////////////////

  cout << "Reading configuration files..."; cout.flush();

  ConfigStreams config(configuration_file);
  if (exists(sec_config_file))
    config.AddFile(sec_config_file);

  config.FindFromBeginning("[domain]");

  config.PeekValue("Ny", Ny);
  config.PeekValue("Nx", Nx);
  
  config.FindFromBeginning("[paths]");

  config.PeekValue("LUC_file", LUC_file);
  Nc = int(file_size(LUC_file)) / sizeof(real) / (Ny * Nx);

  config.PeekValue("Roughness_data_file", roughness_data_file);
  config.PeekValue("Directory_roughness", dir_out);
  config.PeekValue("Roughness_out", file_out);

  cout << " done." << endl;


  ///////////////
  // ROUGHNESS //
  ///////////////

  cout << "Reading roughness data..."; cout.flush();
  FormatFormattedText RoughnessData("<e><e><a>");
  Data<real, 1> roughness_data(Nc);
  RoughnessData.Read(roughness_data_file, "1", roughness_data);
  cout << " done." << endl;
  
  FormatBinary<real> LucFormat ;
 
  Data<real, 3> LUC(Nc, Ny, Nx);
  Data<real, 2> RoughnessValue(Ny, Nx);
  
  RoughnessValue.Fill(0.0);
  LucFormat.Read(LUC_file, LUC);

  for (k=0; k<Nc; k++)
    for (j=0; j<Ny; j++)
      for (i=0; i<Nx; i++)
	RoughnessValue(j, i) += LUC(k, j, i) * roughness_data(k);


  cout << "Writing roughness binary ..."; cout.flush();

  FormatBinary<real> RoughnessFormat;
  RoughnessFormat.Write(RoughnessValue, dir_out + "/" + file_out);
    
  cout << " done." << endl;


  END;
  
  return 0;

}
